# h316_sch

This repository contains schematics for the Honeywell H316 minicomputer
drawn using the [gEDA](http://www.geda-project.org/) project's gschem
schematic capture tool.

The schematics can be printed (or converted to the printable "pdf" format)
but since scanned versions of these schematics are already available on
sites such as [bitsavers](http://www.trailing-edge.com) it is reasonable
to ask what the point is. The schematics can be netlisted into a verilog
gate-level netlist, and this representation can be simulated on a modern
computer. This allows the operation of the computer to be studied in
detail - typically by looking at waveforms. The exercise of getting the
simulation to work correctly has uncovered quite a few mistakes in the
scanned schematics, so it is quite likely that these represent the most
accurate record of the logic design of this machine.

Why would anyone want to study such an old machine in detail? I'd argue
that the Honeywell H316, along with the other machines designed by the
Computer Control Company (CCC), which Honeywell acquired, are important
historical machines. These machines were among the first embedded
computers. Their use as the CPU in the first network router (the
Arpanet IMP) is relatively well known. But they were also widely used in
embedded control applications such as factory automation, traffic light
control, etc. Military versions were also used for such things as the
main engine controllers on the space shuttle.

These machines also deserve recognition from a purely design aesthetic
standpoint. They are exceptionally efficient in their use of hardware and
demonstrate, for example, just what can be achieved if you can afford just
a single adder. Machine registers built out of just two NAND gates. The
forgotten art of latch-based (as opposed to flip-flop based) digital logic
design.

## Copying

This repository is, to borrow the vernacular of soccer punditry, very
much a repository of two halves:

 1. The schematics themselves
 2. The means to process the schematics into a gate-level netlist and
    simulate it.

### Schematics

The schematics, located in the "honeywell" sub-directory are copyright
Honeywell (see below for details).

The intention in making them available in an electronic format that
maintains a representation of the electrical connectivity of the component
gates is to facilitate study of the design.

I have no permission or license to use or publish this copyright work,
and I am certainly in no position to give anyone else a license to do so.
If you take the schematics from the repository and use or publish them
then you are on your own should the copyright owner seek to assert their
rights.

The principal sources for these schematics are:

 * H316 Central Processor Instructions and Logic Diagrams, February 1970.
   Doc. No. 70130072174G. Copyright &copy; 1970, By Honeywell Inc., Computer
   Control Division, Framingham, Massachusetts.

 * Model H316/06 Memory Expansion Option Instruction Manual, June 1973.
   Doc. No. 70130072276H. Copyright &copy; 1970, Honeywell Inc. Copyright
   &copy; 1971, 1972, 1973, Honeywell Information Systems Inc.

 * H316-12 Real Time Clock Option Instruction Manual, May 1969.
   Doc. No. 70130072179A. Copyright &copy; 1970, By Honeywell Inc., Computer
   Control Division, Framingham, Massachusetts.

 * H316-08 Memory Lockout Option Instruction Manual, June 1971.
   Doc. No. 70130072309D. Copyright &copy; 1970, Honeywell Computer Control
   Divsion. Copyright &copy; 1970, 1971 Honeywell Information Systems Inc.

 * H-316-20 Data Multiplex Multiplex (sic) Control Option, June 1975.
   Order No. 70130072180L. Copyright &copy; 1969, 1970, Honeywell Inc.
   Copyright &copy; 1971, 1972, 1973, 1974, Honeywell Information Systems Inc.

### Netlisting

The tools and simulation environment in the "netlist" sub-directory are
copyright Adrian Wise, see individual files for details.

These materials are made available under the terms of the GNU General
Public License version 3. See "netlist/COPYING".

Note that the verilog files are also made available under the GPL, rather
than a hardware-focused open-source license. This is because I do not
believe that there is any practical modern-day implementation technology
that would allow them to be used to produce a physical embodiment of the
design. They are only useful in running a *simulation*, and this is
therefore effectively defining a computer program that is run on a
licensee's computer just like any other GPL program.
