/*
 * makesch - make a skeleton schematic from text file
 *
 * Copyright (C) 2006, 2017  Adrian Wise
 *
 * This file is part of makesch.
 *
 * makesch is free software: you can redistribute it and/or modify
 * it under the terms of version 3 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See http://www.series16.adrianwise.co.uk
 *   mailto:apw@adrianwise.co.uk
 *
 */
class Text
{
public:
  Text(int x, int y, int color, int size, int visibility,
       int show_name_value, int angle, int alignment, const char *str);
  void output(int x0, int y0, char *attribute);
private:
  int x;
  int y;
  int color;
  int size;
  int visibility;
  int show_name_value;
  int angle;
  int alignment;
  const char *str;
  
  char *fix_attribute(char *attribute);
};


class Pin
{
public:
  Pin(int x1, int y1, int x2, int y2, int color);
  void annotate(int xx, int yy, int x, int y, char *signal, char *page,
                char *post, bool ff);
  
private:
  int x1;
  int y1;
  int x2;
  int y2;
  int color;
};

class Component
{
public:
  enum TYPE
  {
    TITLE,
    GATE
  };

  Component(const char *name, const char *nickname, enum TYPE type, int x, int y,
            int num_texts, Text texts[],
            int num_pins, Pin pins[]);
  void instance(int xx, int yy, char **attributes);
  void annotate(int xx, int yy, int pin_no, char *signal,
                char *page, char *post);
  int get_num_pins() const {return num_pins;};
  int get_num_texts() const {return num_texts;};
  enum TYPE get_type() const {return type;};

  static Component *lookup(const char *nickname);

private:
  const char *name;
  const char *nickname;
  enum TYPE type;
  int x;
  int y;
  int num_texts;
  Text **texts;
  int num_pins;
  Pin **pins;

  static Component *components[];
};
