/***********************************************************************
 *
 * h316 - H316 mainframe
 * 
 * Copyright (C) 2007, 2011, 2017 Adrian Wise
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of version 3 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See http://www.series16.adrianwise.co.uk
 *   mailto:apw@adrianwise.co.uk
 * 
 ***********************************************************************
 * 
 * This is a wrapper around the gate-level netlists of the CPU, MLO,
 * SDMC, and the memory expansion 3x1 bloc combining the various input
 * and output buses into verilog buses.
 * 
 **********************************************************************/
module h316_mfm_bus (/*AUTOARG*/
   // Outputs
   LMPRN_n, INDxx_n, OTBxx_p, ADBxx_n, SMKXX_n, OCPLS_n, SMK01_n,
   ERLXX_n, CMKXX_n, ENDOP_n, RRLIN_n, MxxFF_p, MADxx_n, BANKD_n,
   BANKC_n, BANKB_n, WRITE_n, MBSYX_p, BANKA_n, MADCL_n_A, STROB_n,
   MADCL_n_C, XTIMG_p, YTIMG_p, MDOxx_p, xMADxx_n, xBANKD_n, xBANKC_n,
   xBANKB_n, xWRITE_n, MBUSY_p, xBANKA_n, MARRC_n_A, MSTRB_n,
   MARRC_n_B, MXTMG_p, MYTMG_p, DALxx_n,
   // Inouts
   SENSx_p, SENSx_n, DRLIN_n, PIL00_n,
   // Inputs
   PFINT_n, MSTCL_n, ENTRA_n, ENTRM_n, FETCH_n, RUNMD_n, CLBMP_n,
   CLPMP_n, START_p, DSPLO_p, DSPLY_p, ENTRB_n, ENTRP_n, MEMAC_n,
   STEPP_n, READY_n, CLAMP_n, CLMMP_n, MASTO_n, DSPLM_p, DSPLA_p,
   DSPLB_p, SDRxx_n, INBxx_n, PARCK_n, MMxxF_n, MMDxx_n, DILxx_p
   );

   // Power supply
   input         PFINT_n ;

   // Console
   input         MSTCL_n ;
   input         ENTRA_n ;
   input         ENTRM_n ;
   input         FETCH_n ;
   input         RUNMD_n ;
   input         CLBMP_n ;
   input         CLPMP_n ;
   inout  [1:4]  SENSx_p ;
   inout  [1:4]  SENSx_n ;
   input         START_p ;
   input         DSPLO_p ;
   input         DSPLY_p ;
   input         ENTRB_n ;
   input         ENTRP_n ;
   input         MEMAC_n ;
   input         STEPP_n ;
   input         READY_n ;
   input         CLAMP_n ;
   input         CLMMP_n ;
   input         MASTO_n ;
   output        LMPRN_n ;
   input         DSPLM_p ;
   input         DSPLA_p ;
   input         DSPLB_p ;
   input  [1:16] SDRxx_n ;
   output [1:16] INDxx_n ;

   // I/O Bus
   output [1:16] OTBxx_p ;
   input  [1:16] INBxx_n ;
   output [7:16] ADBxx_n ;
   inout         DRLIN_n ; // I/O because RTC drives this internally
   output        SMKXX_n ;
   output        OCPLS_n ;
   output        SMK01_n ;
   output        ERLXX_n ;
   input         PARCK_n ;
   inout         PIL00_n ; // I/O because of start-button & RTC interrupt
   output        CMKXX_n ;
   output        ENDOP_n ;
   output        RRLIN_n ;

   // Standard Memory
   output [1:16] MxxFF_p ;
   input  [1:16] MMxxF_n ;
   output [5:16] MADxx_n ;
   output        BANKD_n ;
   output        BANKC_n ;
   output        BANKB_n ;
   output        WRITE_n ;
   output        MBSYX_p ;
   output        BANKA_n ;
   output        MADCL_n_A ;
   output        STROB_n ;
   output        MADCL_n_C ;
   output        XTIMG_p ;
   output        YTIMG_p ;
   
   // Memory Expansion
   output [1:16] MDOxx_p ;
   input [ 1:16] MMDxx_n ;
   output [5:16] xMADxx_n ;
   output        xBANKD_n ;
   output        xBANKC_n ;
   output        xBANKB_n ;
   output        xWRITE_n;
   output        MBUSY_p;
   output        xBANKA_n ;
   output        MARRC_n_A;
   output        MSTRB_n;
   output        MARRC_n_B;
   output        MXTMG_p;
   output        MYTMG_p;

   // DMC
   input [1:16] DILxx_p ;
   output [1:16] DALxx_n ;

   // Inputs from PI option to CPU?
   wire [1:7]  IADXx_n ;
   wire SEXRQ_p ;
   wire PIREQ_p ;
   wire IG067_p ;
   wire IG107_p ;
   wire IG127_p ;

   wire IG117_p ;

   wire IG077_n ;
   wire IG117_n ;
   wire IG137_n ;
   
   pullup pu_IADX1_n  (IADXx_n[1]);
   pullup pu_IADX2_n  (IADXx_n[2]);
   pullup pu_IADX3_n  (IADXx_n[3]);
   pullup pu_IADX4_n  (IADXx_n[4]);
   pullup pu_IADX5_n  (IADXx_n[5]);
   pullup pu_IADX6_n  (IADXx_n[6]);
   pullup pu_IADX7_n  (IADXx_n[7]);

   assign SEXRQ_p = IG063_n;
   assign PIREQ_p = IG063_n;
   
   /*
    * Notes from LBD 0.135:
    * 
    * 1: THIS PT IS GND WITH NO PI OPTION
    *    (referring to IG067+, IG107+, and IG127+)
    */
   assign IG067_p = 1'b0;
   assign IG107_p = 1'b0;
   assign IG127_p = 1'b0;

   /*
    * Other inputs from PI float, so add a pullup
    */
   pullup pu_IG117_p (IG117_p);
   pullup pu_IG077_n (IG077_n);
   pullup pu_IG117_n (IG117_n);
   pullup pu_IG137_n (IG137_n);
   
   
   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire                 ACYLF_n;                // From cpu of h316_cpu.v
   wire                 ACYLF_p;                // From cpu of h316_cpu.v
   wire                 BREAK_n;                // From cpu of h316_cpu.v
   wire                 BSD02_p;                // From cpu of h316_cpu.v
   wire                 CLMTR_n;                // From dmc of h316_sdmc.v
   wire                 CLPIL_n;                // From cpu of h316_cpu.v
   wire                 CLSEX_n_A;              // From cpu of h316_cpu.v
   wire                 CLSEX_n_B;              // From cpu of h316_cpu.v
   wire                 CLSEX_n_C;              // From cpu of h316_cpu.v
   wire                 D03FF_n;                // From cpu of h316_cpu.v
   wire                 D04FF_n;                // From cpu of h316_cpu.v
   wire                 D05FF_n;                // From cpu of h316_cpu.v
   wire                 D06FF_n;                // From cpu of h316_cpu.v
   wire                 D07FF_n;                // From cpu of h316_cpu.v
   wire                 DCY2X_p;                // From dmc of h316_sdmc.v
   wire                 DIVOP_n;                // From cpu of h316_cpu.v
   wire                 DMCCY_n;                // From dmc of h316_sdmc.v
   wire                 DMCWR_n;                // From dmc of h316_sdmc.v
   wire                 DMCYQ_n;                // From dmc of h316_sdmc.v
   wire                 DMRRL_p;                // From dmc of h316_sdmc.v
   wire                 DREDY_n;                // From mex of h316_mex.v
   wire                 EIYHS_p;                // From cpu of h316_cpu.v
   wire                 EIYTS_n_A;              // From dmc of h316_sdmc.v
   wire                 EOINS_n;                // From cpu of h316_cpu.v
   wire                 EPARB_p;                // From mex of h316_mex.v
   wire                 GENOP_n;                // From dmc of h316_sdmc.v
   wire                 ICYLF_p;                // From cpu of h316_cpu.v
   wire                 IG063_n;                // From cpu of h316_cpu.v
   wire                 IG063_p;                // From cpu of h316_cpu.v
   wire                 IOACY_p;                // From dmc of h316_sdmc.v
   wire                 IOGRP_n;                // From cpu of h316_cpu.v
   wire                 IRSOP_p;                // From cpu of h316_cpu.v
   wire                 IYB09_p;                // To/From cpu of h316_cpu.v, ...
   wire                 IYB10_p;                // To/From cpu of h316_cpu.v, ...
   wire                 IYB11_p;                // To/From cpu of h316_cpu.v, ...
   wire                 IYB12_p;                // To/From cpu of h316_cpu.v, ...
   wire                 IYB13_p;                // To/From cpu of h316_cpu.v, ...
   wire                 IYB14_p;                // To/From cpu of h316_cpu.v, ...
   wire                 IYB15_p;                // To/From cpu of h316_cpu.v, ...
   wire                 IYB16_p;                // To/From cpu of h316_cpu.v, ...
   wire                 M01FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M02FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M03FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M04FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M05FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M06FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M07FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M08FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M09FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M10FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M11FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M12FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M13FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M14FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M15FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 M16FF_n;                // To/From cpu of h316_cpu.v, ...
   wire                 MAI02_p;                // From cpu of h316_cpu.v
   wire                 MAI03_p;                // From cpu of h316_cpu.v
   wire                 MAI04_p;                // From cpu of h316_cpu.v
   wire                 MAI05_p;                // From cpu of h316_cpu.v
   wire                 MAI06_p;                // From cpu of h316_cpu.v
   wire                 MAI07_p;                // From cpu of h316_cpu.v
   wire                 MAI08_p;                // From cpu of h316_cpu.v
   wire                 MAI09_p;                // From cpu of h316_cpu.v
   wire                 MAI10_p;                // From cpu of h316_cpu.v
   wire                 MAI11_p;                // From cpu of h316_cpu.v
   wire                 MAI12_p;                // From cpu of h316_cpu.v
   wire                 MAI13_p;                // From cpu of h316_cpu.v
   wire                 MAI14_p;                // From cpu of h316_cpu.v
   wire                 MAI15_p;                // From cpu of h316_cpu.v
   wire                 MAI16_p;                // From cpu of h316_cpu.v
   wire                 MCRST_p;                // From cpu of h316_cpu.v
   wire                 MCSET_p;                // From cpu of h316_cpu.v
   wire                 MCTLG_p;                // From cpu of h316_cpu.v
   wire                 MDI01_p;                // From cpu of h316_cpu.v
   wire                 MDI02_p;                // From cpu of h316_cpu.v
   wire                 MDI03_p;                // From cpu of h316_cpu.v
   wire                 MDI04_p;                // From cpu of h316_cpu.v
   wire                 MDI05_p;                // From cpu of h316_cpu.v
   wire                 MDI06_p;                // From cpu of h316_cpu.v
   wire                 MDI07_p;                // From cpu of h316_cpu.v
   wire                 MDI08_p;                // From cpu of h316_cpu.v
   wire                 MDI09_p;                // From cpu of h316_cpu.v
   wire                 MDI10_p;                // From cpu of h316_cpu.v
   wire                 MDI11_p;                // From cpu of h316_cpu.v
   wire                 MDI12_p;                // From cpu of h316_cpu.v
   wire                 MDI13_p;                // From cpu of h316_cpu.v
   wire                 MDI14_p;                // From cpu of h316_cpu.v
   wire                 MDI15_p;                // From cpu of h316_cpu.v
   wire                 MDI16_p;                // From cpu of h316_cpu.v
   wire                 MDO01_p_A;              // From mex of h316_mex.v
   wire                 MDO02_p_A;              // From mex of h316_mex.v
   wire                 MDO03_p_A;              // From mex of h316_mex.v
   wire                 MDO04_p_A;              // From mex of h316_mex.v
   wire                 MDO05_p_A;              // From mex of h316_mex.v
   wire                 MDO06_p_A;              // From mex of h316_mex.v
   wire                 MDO07_p_A;              // From mex of h316_mex.v
   wire                 MDO08_p_A;              // From mex of h316_mex.v
   wire                 MDO09_p_A;              // From mex of h316_mex.v
   wire                 MDO10_p_A;              // From mex of h316_mex.v
   wire                 MDO11_p_A;              // From mex of h316_mex.v
   wire                 MDO12_p_A;              // From mex of h316_mex.v
   wire                 MDO13_p_A;              // From mex of h316_mex.v
   wire                 MDO14_p_A;              // From mex of h316_mex.v
   wire                 MDO15_p_A;              // From mex of h316_mex.v
   wire                 MDO16_p_A;              // From mex of h316_mex.v
   wire                 MEMCI_p;                // From cpu of h316_cpu.v
   wire                 MPYOP_n;                // From cpu of h316_cpu.v
   wire                 MREAD_p;                // From cpu of h316_cpu.v
   wire                 NRMOP_p;                // From cpu of h316_cpu.v
   wire                 NRMOP_p_D;              // From dmc of h316_sdmc.v
   wire                 OBINH_n;                // From dmc of h316_sdmc.v
   wire                 PAGEO_p;                // From mex of h316_mex.v
   wire                 PAMEO_p;                // From mex of h316_mex.v
   wire                 PBNKA_p;                // From cpu of h316_cpu.v
   wire                 PBNKB_p;                // From cpu of h316_cpu.v
   wire                 PBNKC_p;                // From cpu of h316_cpu.v
   wire                 PBNKD_p;                // From cpu of h316_cpu.v
   wire                 PROTX_p;                // From mlo of h316_mlo.v
   wire                 RPTT2_p;                // To/From cpu of h316_cpu.v, ...
   wire                 RRCXX_n;                // From dmc of h316_sdmc.v
   wire                 RRCXX_p;                // From cpu of h316_cpu.v
   wire                 RTCCL_n;                // From dmc of h316_sdmc.v
   wire                 RTCEN_n_A;              // From dmc of h316_sdmc.v
   wire                 SCZR0_p;                // From cpu of h316_cpu.v
   wire                 SETTA_n;                // From dmc of h316_sdmc.v
   wire                 SMK20_n;                // From cpu of h316_cpu.v
   wire                 SMPIL_p_A;              // From cpu of h316_cpu.v
   wire                 SMPIL_p_B;              // From cpu of h316_cpu.v
   wire                 SMPIL_p_C;              // From cpu of h316_cpu.v
   wire                 T2DMC_n;                // From dmc of h316_sdmc.v
   wire                 T2DMC_p;                // From dmc of h316_sdmc.v
   wire                 TL1FF_p_A;              // From cpu of h316_cpu.v
   wire                 TL23F_n;                // From cpu of h316_cpu.v
   wire                 TL24F_p;                // From cpu of h316_cpu.v
   wire                 TL2FF_p_B;              // From cpu of h316_cpu.v
   wire                 TL3FF_n;                // From cpu of h316_cpu.v
   wire                 TL3FF_p_A;              // From cpu of h316_cpu.v
   wire                 TL4FF_p_A;              // From cpu of h316_cpu.v
   wire                 TLBFF_n;                // From dmc of h316_sdmc.v
   wire                 Y02FF_n;                // From dmc of h316_sdmc.v
   wire                 Y03FF_n;                // From dmc of h316_sdmc.v
   wire                 Y04FF_n;                // From dmc of h316_sdmc.v
   wire                 Y05FF_n;                // From dmc of h316_sdmc.v
   wire                 Y06FF_n;                // From dmc of h316_sdmc.v
   wire                 Y07FF_n;                // From dmc of h316_sdmc.v
   wire                 Y08FF_n;                // From dmc of h316_sdmc.v
   // End of automatics
   
    /*
    * Notes from LBD 0.142:
    * 
    * 2: WITH NO PI TIE SEXRQ+ AND PIREQ+ TO IG063-
    */
   assign SEXRQ_p = IG063_n;
   assign PIREQ_p = IG063_n;
   
   /*
    * Notes from LBD 0.135:
    * 
    * 1: THIS PT IS GND WITH NO PI OPTION
    *    (referring to IG067+, IG107+, and IG127+)
    */
   assign IG067_p = 1'b0;
   assign IG107_p = 1'b0;
   assign IG127_p = 1'b0;

   h316_cpu cpu (// Outputs
                 .ADB07_n               (ADBxx_n[7]),
                 .ADB08_n               (ADBxx_n[8]),
                 .ADB09_n               (ADBxx_n[9]),
                 .ADB10_n               (ADBxx_n[10]),
                 .ADB11_n               (ADBxx_n[11]),
                 .ADB12_n               (ADBxx_n[12]),
                 .ADB13_n               (ADBxx_n[13]),
                 .ADB14_n               (ADBxx_n[14]),
                 .ADB15_n               (ADBxx_n[15]),
                 .ADB16_n               (ADBxx_n[16]),
                 .IND01_n               (INDxx_n[1]),
                 .IND02_n               (INDxx_n[2]),
                 .IND03_n               (INDxx_n[3]),
                 .IND04_n               (INDxx_n[4]),
                 .IND05_n               (INDxx_n[5]),
                 .IND06_n               (INDxx_n[6]),
                 .IND07_n               (INDxx_n[7]),
                 .IND08_n               (INDxx_n[8]),
                 .IND09_n               (INDxx_n[9]),
                 .IND10_n               (INDxx_n[10]),
                 .IND11_n               (INDxx_n[11]),
                 .IND12_n               (INDxx_n[12]),
                 .IND13_n               (INDxx_n[13]),
                 .IND14_n               (INDxx_n[14]),
                 .IND15_n               (INDxx_n[15]),
                 .IND16_n               (INDxx_n[16]),
                 .M01FF_p               (MxxFF_p[1]),
                 .M02FF_p               (MxxFF_p[2]),
                 .M03FF_p               (MxxFF_p[3]),
                 .M04FF_p               (MxxFF_p[4]),
                 .M05FF_p               (MxxFF_p[5]),
                 .M06FF_p               (MxxFF_p[6]),
                 .M07FF_p               (MxxFF_p[7]),
                 .M08FF_p               (MxxFF_p[8]),
                 .M09FF_p               (MxxFF_p[9]),
                 .M10FF_p               (MxxFF_p[10]),
                 .M11FF_p               (MxxFF_p[11]),
                 .M12FF_p               (MxxFF_p[12]),
                 .M13FF_p               (MxxFF_p[13]),
                 .M14FF_p               (MxxFF_p[14]),
                 .M15FF_p               (MxxFF_p[15]),
                 .M16FF_p               (MxxFF_p[16]),
                 .MAD05_n               (MADxx_n[5]),
                 .MAD06_n               (MADxx_n[6]),
                 .MAD07_n               (MADxx_n[7]),
                 .MAD08_n               (MADxx_n[8]),
                 .MAD09_n               (MADxx_n[9]),
                 .MAD10_n               (MADxx_n[10]),
                 .MAD11_n               (MADxx_n[11]),
                 .MAD12_n               (MADxx_n[12]),
                 .MAD13_n               (MADxx_n[13]),
                 .MAD14_n               (MADxx_n[14]),
                 .MAD15_n               (MADxx_n[15]),
                 .MAD16_n               (MADxx_n[16]),
                 .OTB01_p               (OTBxx_p[1]),
                 .OTB02_p               (OTBxx_p[2]),
                 .OTB03_p               (OTBxx_p[3]),
                 .OTB04_p               (OTBxx_p[4]),
                 .OTB05_p               (OTBxx_p[5]),
                 .OTB06_p               (OTBxx_p[6]),
                 .OTB07_p               (OTBxx_p[7]),
                 .OTB08_p               (OTBxx_p[8]),
                 .OTB09_p               (OTBxx_p[9]),
                 .OTB10_p               (OTBxx_p[10]),
                 .OTB11_p               (OTBxx_p[11]),
                 .OTB12_p               (OTBxx_p[12]),
                 .OTB13_p               (OTBxx_p[13]),
                 .OTB14_p               (OTBxx_p[14]),
                 .OTB15_p               (OTBxx_p[15]),
                 .OTB16_p               (OTBxx_p[16]),
                 .MEM01_p               (MDI01_p),
                 .MEM02_p               (MDI02_p),
                 .MEM03_p               (MDI03_p),
                 .MEM04_p               (MDI04_p),
                 .MEM05_p               (MDI05_p),
                 .MEM06_p               (MDI06_p),
                 .MEM07_p               (MDI07_p),
                 .MEM08_p               (MDI08_p),
                 .MEM09_p               (MDI09_p),
                 .MEM10_p               (MDI10_p),
                 .MEM11_p               (MDI11_p),
                 .MEM12_p               (MDI12_p),
                 .MEM13_p               (MDI13_p),
                 .MEM14_p               (MDI14_p),
                 .MEM15_p               (MDI15_p),
                 .MEM16_p               (MDI16_p),
                 .M02AD_p               (MAI02_p),
                 .M03AD_p               (MAI03_p),
                 .M04AD_p               (MAI04_p),
                 .M05AD_p               (MAI05_p),
                 .M06AD_p               (MAI06_p),
                 .M07AD_p               (MAI07_p),
                 .M08AD_p               (MAI08_p),
                 .M09AD_p               (MAI09_p),
                 .M10AD_p               (MAI10_p),
                 .M11AD_p               (MAI11_p),
                 .M12AD_p               (MAI12_p),
                 .M13AD_p               (MAI13_p),
                 .M14AD_p               (MAI14_p),
                 .M15AD_p               (MAI15_p),
                 .M16AD_p               (MAI16_p),
                 .PBNKA_p               (PBNKA_p),
                 .PBNKB_p               (PBNKB_p),
                 .PBNKC_p               (PBNKC_p),
                 .PBNKD_p               (PBNKD_p),
                 .MREAD_p               (MREAD_p),
                 .MMCYI_p               (MEMCI_p),
                 // Inputs
                 .DMCLY_n               (EIYTS_n_A),
                 .DMCCI_n               (TLBFF_n),
                 .IADX1_n               (IADXx_n[1]),
                 .IADX2_n               (IADXx_n[2]),
                 .IADX3_n               (IADXx_n[3]),
                 .IADX4_n               (IADXx_n[4]),
                 .IADX5_n               (IADXx_n[5]),
                 .IADX6_n               (IADXx_n[6]),
                 .IADX7_n               (IADXx_n[7]),
                 .INB01_n               (INBxx_n[1]),
                 .INB02_n               (INBxx_n[2]),
                 .INB03_n               (INBxx_n[3]),
                 .INB04_n               (INBxx_n[4]),
                 .INB05_n               (INBxx_n[5]),
                 .INB06_n               (INBxx_n[6]),
                 .INB07_n               (INBxx_n[7]),
                 .INB08_n               (INBxx_n[8]),
                 .INB09_n               (INBxx_n[9]),
                 .INB10_n               (INBxx_n[10]),
                 .INB11_n               (INBxx_n[11]),
                 .INB12_n               (INBxx_n[12]),
                 .INB13_n               (INBxx_n[13]),
                 .INB14_n               (INBxx_n[14]),
                 .INB15_n               (INBxx_n[15]),
                 .INB16_n               (INBxx_n[16]),
                 .MM01F_n               (MMxxF_n[1]),
                 .MM02F_n               (MMxxF_n[2]),
                 .MM03F_n               (MMxxF_n[3]),
                 .MM04F_n               (MMxxF_n[4]),
                 .MM05F_n               (MMxxF_n[5]),
                 .MM06F_n               (MMxxF_n[6]),
                 .MM07F_n               (MMxxF_n[7]),
                 .MM08F_n               (MMxxF_n[8]),
                 .MM09F_n               (MMxxF_n[9]),
                 .MM10F_n               (MMxxF_n[10]),
                 .MM11F_n               (MMxxF_n[11]),
                 .MM12F_n               (MMxxF_n[12]),
                 .MM13F_n               (MMxxF_n[13]),
                 .MM14F_n               (MMxxF_n[14]),
                 .MM15F_n               (MMxxF_n[15]),
                 .MM16F_n               (MMxxF_n[16]),
                 .SDR01_n               (SDRxx_n[1]),
                 .SDR02_n               (SDRxx_n[2]),
                 .SDR03_n               (SDRxx_n[3]),
                 .SDR04_n               (SDRxx_n[4]),
                 .SDR05_n               (SDRxx_n[5]),
                 .SDR06_n               (SDRxx_n[6]),
                 .SDR07_n               (SDRxx_n[7]),
                 .SDR08_n               (SDRxx_n[8]),
                 .SDR09_n               (SDRxx_n[9]),
                 .SDR10_n               (SDRxx_n[10]),
                 .SDR11_n               (SDRxx_n[11]),
                 .SDR12_n               (SDRxx_n[12]),
                 .SDR13_n               (SDRxx_n[13]),
                 .SDR14_n               (SDRxx_n[14]),
                 .SDR15_n               (SDRxx_n[15]),
                 .SDR16_n               (SDRxx_n[16]),
                 .SENS1_p               (SENSx_p[1]),
                 .SENS1_n               (SENSx_n[1]),
                 .SENS2_p               (SENSx_p[2]),
                 .SENS2_n               (SENSx_n[2]),
                 .SENS3_p               (SENSx_p[3]),
                 .SENS3_n               (SENSx_n[3]),
                 .SENS4_p               (SENSx_p[4]),
                 .SENS4_n               (SENSx_n[4]),
                 .MMD01_p               (MDO01_p_A),
                 .MMD02_p               (MDO02_p_A),
                 .MMD03_p               (MDO03_p_A),
                 .MMD04_p               (MDO04_p_A),
                 .MMD05_p               (MDO05_p_A),
                 .MMD06_p               (MDO06_p_A),
                 .MMD07_p               (MDO07_p_A),
                 .MMD08_p               (MDO08_p_A),
                 .MMD09_p               (MDO09_p_A),
                 .MMD10_p               (MDO10_p_A),
                 .MMD11_p               (MDO11_p_A),
                 .MMD12_p               (MDO12_p_A),
                 .MMD13_p               (MDO13_p_A),
                 .MMD14_p               (MDO14_p_A),
                 .MMD15_p               (MDO15_p_A),
                 .MMD16_p               (MDO16_p_A),
                 /*AUTOINST*/
                 // Outputs
                 .ACYLF_p               (ACYLF_p),
                 .ACYLF_n               (ACYLF_n),
                 .BANKA_n               (BANKA_n),
                 .BANKB_n               (BANKB_n),
                 .BANKC_n               (BANKC_n),
                 .BANKD_n               (BANKD_n),
                 .BREAK_n               (BREAK_n),
                 .BSD02_p               (BSD02_p),
                 .CLPIL_n               (CLPIL_n),
                 .CLSEX_n_A             (CLSEX_n_A),
                 .CLSEX_n_B             (CLSEX_n_B),
                 .CLSEX_n_C             (CLSEX_n_C),
                 .CMKXX_n               (CMKXX_n),
                 .D03FF_n               (D03FF_n),
                 .D04FF_n               (D04FF_n),
                 .D05FF_n               (D05FF_n),
                 .D06FF_n               (D06FF_n),
                 .D07FF_n               (D07FF_n),
                 .DIVOP_n               (DIVOP_n),
                 .EIYHS_p               (EIYHS_p),
                 .EOINS_n               (EOINS_n),
                 .ICYLF_p               (ICYLF_p),
                 .IG063_p               (IG063_p),
                 .IG063_n               (IG063_n),
                 .IOGRP_n               (IOGRP_n),
                 .IRSOP_p               (IRSOP_p),
                 .LMPRN_n               (LMPRN_n),
                 .MADCL_n_A             (MADCL_n_A),
                 .MADCL_n_C             (MADCL_n_C),
                 .MBSYX_p               (MBSYX_p),
                 .MCRST_p               (MCRST_p),
                 .MCSET_p               (MCSET_p),
                 .MCTLG_p               (MCTLG_p),
                 .MPYOP_n               (MPYOP_n),
                 .NRMOP_p               (NRMOP_p),
                 .OCPLS_n               (OCPLS_n),
                 .RRCXX_p               (RRCXX_p),
                 .RRLIN_n               (RRLIN_n),
                 .SCZR0_p               (SCZR0_p),
                 .SMK01_n               (SMK01_n),
                 .SMK20_n               (SMK20_n),
                 .SMKXX_n               (SMKXX_n),
                 .SMPIL_p_A             (SMPIL_p_A),
                 .SMPIL_p_B             (SMPIL_p_B),
                 .SMPIL_p_C             (SMPIL_p_C),
                 .STROB_n               (STROB_n),
                 .TL1FF_p_A             (TL1FF_p_A),
                 .TL23F_n               (TL23F_n),
                 .TL24F_p               (TL24F_p),
                 .TL2FF_p_B             (TL2FF_p_B),
                 .TL3FF_p_A             (TL3FF_p_A),
                 .TL3FF_n               (TL3FF_n),
                 .TL4FF_p_A             (TL4FF_p_A),
                 .WRITE_n               (WRITE_n),
                 .XTIMG_p               (XTIMG_p),
                 .YTIMG_p               (YTIMG_p),
                 // Inouts
                 .CLMTR_n               (CLMTR_n),
                 .DRLIN_n               (DRLIN_n),
                 .EIYTS_n_A             (EIYTS_n_A),
                 .GENOP_n               (GENOP_n),
                 .IOACY_p               (IOACY_p),
                 .IYB09_p               (IYB09_p),
                 .IYB10_p               (IYB10_p),
                 .IYB11_p               (IYB11_p),
                 .IYB12_p               (IYB12_p),
                 .IYB13_p               (IYB13_p),
                 .IYB14_p               (IYB14_p),
                 .IYB15_p               (IYB15_p),
                 .IYB16_p               (IYB16_p),
                 .M01FF_n               (M01FF_n),
                 .M02FF_n               (M02FF_n),
                 .M03FF_n               (M03FF_n),
                 .M04FF_n               (M04FF_n),
                 .M05FF_n               (M05FF_n),
                 .M06FF_n               (M06FF_n),
                 .M07FF_n               (M07FF_n),
                 .M08FF_n               (M08FF_n),
                 .M09FF_n               (M09FF_n),
                 .M10FF_n               (M10FF_n),
                 .M11FF_n               (M11FF_n),
                 .M12FF_n               (M12FF_n),
                 .M13FF_n               (M13FF_n),
                 .M14FF_n               (M14FF_n),
                 .M15FF_n               (M15FF_n),
                 .M16FF_n               (M16FF_n),
                 .PIL00_n               (PIL00_n),
                 .RPTT2_p               (RPTT2_p),
                 .RRCXX_n               (RRCXX_n),
                 .Y02FF_n               (Y02FF_n),
                 .Y03FF_n               (Y03FF_n),
                 .Y04FF_n               (Y04FF_n),
                 .Y05FF_n               (Y05FF_n),
                 .Y06FF_n               (Y06FF_n),
                 .Y07FF_n               (Y07FF_n),
                 .Y08FF_n               (Y08FF_n),
                 // Inputs
                 .CLAMP_n               (CLAMP_n),
                 .CLBMP_n               (CLBMP_n),
                 .CLMMP_n               (CLMMP_n),
                 .CLPMP_n               (CLPMP_n),
                 .DCY2X_p               (DCY2X_p),
                 .DMCCY_n               (DMCCY_n),
                 .DMCWR_n               (DMCWR_n),
                 .DMCYQ_n               (DMCYQ_n),
                 .DMRRL_p               (DMRRL_p),
                 .DREDY_n               (DREDY_n),
                 .DSPLA_p               (DSPLA_p),
                 .DSPLB_p               (DSPLB_p),
                 .DSPLM_p               (DSPLM_p),
                 .DSPLO_p               (DSPLO_p),
                 .DSPLY_p               (DSPLY_p),
                 .ENTRA_n               (ENTRA_n),
                 .ENTRB_n               (ENTRB_n),
                 .ENTRM_n               (ENTRM_n),
                 .ENTRP_n               (ENTRP_n),
                 .EPARB_p               (EPARB_p),
                 .FETCH_n               (FETCH_n),
                 .IG067_p               (IG067_p),
                 .IG077_n               (IG077_n),
                 .IG107_p               (IG107_p),
                 .IG117_p               (IG117_p),
                 .IG117_n               (IG117_n),
                 .IG127_p               (IG127_p),
                 .IG137_n               (IG137_n),
                 .MASTO_n               (MASTO_n),
                 .MEMAC_n               (MEMAC_n),
                 .MSTCL_n               (MSTCL_n),
                 .NRMOP_p_D             (NRMOP_p_D),
                 .OBINH_n               (OBINH_n),
                 .PAGEO_p               (PAGEO_p),
                 .PAMEO_p               (PAMEO_p),
                 .PARCK_n               (PARCK_n),
                 .PFINT_n               (PFINT_n),
                 .PIREQ_p               (PIREQ_p),
                 .PROTX_p               (PROTX_p),
                 .READY_n               (READY_n),
                 .RTCCL_n               (RTCCL_n),
                 .RTCEN_n_A             (RTCEN_n_A),
                 .RUNMD_n               (RUNMD_n),
                 .SETTA_n               (SETTA_n),
                 .SEXRQ_p               (SEXRQ_p),
                 .START_p               (START_p),
                 .STEPP_n               (STEPP_n),
                 .T2DMC_p               (T2DMC_p),
                 .T2DMC_n               (T2DMC_n));

   h316_mex mex (// Outputs
                 .BANKA_n               (xBANKA_n),
                 .BANKB_n               (xBANKB_n),
                 .BANKC_n               (xBANKC_n),
                 .BANKD_n               (xBANKD_n),
                 .MAD05_n               (xMADxx_n[5]),
                 .MAD06_n               (xMADxx_n[6]),
                 .MAD07_n               (xMADxx_n[7]),
                 .MAD08_n               (xMADxx_n[8]),
                 .MAD09_n               (xMADxx_n[9]),
                 .MAD10_n               (xMADxx_n[10]),
                 .MAD11_n               (xMADxx_n[11]),
                 .MAD12_n               (xMADxx_n[12]),
                 .MAD13_n               (xMADxx_n[13]),
                 .MAD14_n               (xMADxx_n[14]),
                 .MAD15_n               (xMADxx_n[15]),
                 .MAD16_n               (xMADxx_n[16]),
                 .MDO01_p               (MDOxx_p[1]),
                 .MDO02_p               (MDOxx_p[2]),
                 .MDO03_p               (MDOxx_p[3]),
                 .MDO04_p               (MDOxx_p[4]),
                 .MDO05_p               (MDOxx_p[5]),
                 .MDO06_p               (MDOxx_p[6]),
                 .MDO07_p               (MDOxx_p[7]),
                 .MDO08_p               (MDOxx_p[8]),
                 .MDO09_p               (MDOxx_p[9]),
                 .MDO10_p               (MDOxx_p[10]),
                 .MDO11_p               (MDOxx_p[11]),
                 .MDO12_p               (MDOxx_p[12]),
                 .MDO13_p               (MDOxx_p[13]),
                 .MDO14_p               (MDOxx_p[14]),
                 .MDO15_p               (MDOxx_p[15]),
                 .MDO16_p               (MDOxx_p[16]),
                 .WRITE_n               (xWRITE_n),
                 // Inputs
                 .MMD01_n               (MMDxx_n[1]),
                 .MMD02_n               (MMDxx_n[2]),
                 .MMD03_n               (MMDxx_n[3]),
                 .MMD04_n               (MMDxx_n[4]),
                 .MMD05_n               (MMDxx_n[5]),
                 .MMD06_n               (MMDxx_n[6]),
                 .MMD07_n               (MMDxx_n[7]),
                 .MMD08_n               (MMDxx_n[8]),
                 .MMD09_n               (MMDxx_n[9]),
                 .MMD10_n               (MMDxx_n[10]),
                 .MMD11_n               (MMDxx_n[11]),
                 .MMD12_n               (MMDxx_n[12]),
                 .MMD13_n               (MMDxx_n[13]),
                 .MMD14_n               (MMDxx_n[14]),
                 .MMD15_n               (MMDxx_n[15]),
                 .MMD16_n               (MMDxx_n[16]),
                 /*AUTOINST*/
                 // Outputs
                 .DREDY_n               (DREDY_n),
                 .EPARB_p               (EPARB_p),
                 .MARRC_n_A             (MARRC_n_A),
                 .MARRC_n_B             (MARRC_n_B),
                 .MBUSY_p               (MBUSY_p),
                 .MDO01_p_A             (MDO01_p_A),
                 .MDO02_p_A             (MDO02_p_A),
                 .MDO03_p_A             (MDO03_p_A),
                 .MDO04_p_A             (MDO04_p_A),
                 .MDO05_p_A             (MDO05_p_A),
                 .MDO06_p_A             (MDO06_p_A),
                 .MDO07_p_A             (MDO07_p_A),
                 .MDO08_p_A             (MDO08_p_A),
                 .MDO09_p_A             (MDO09_p_A),
                 .MDO10_p_A             (MDO10_p_A),
                 .MDO11_p_A             (MDO11_p_A),
                 .MDO12_p_A             (MDO12_p_A),
                 .MDO13_p_A             (MDO13_p_A),
                 .MDO14_p_A             (MDO14_p_A),
                 .MDO15_p_A             (MDO15_p_A),
                 .MDO16_p_A             (MDO16_p_A),
                 .MSTRB_n               (MSTRB_n),
                 .MXTMG_p               (MXTMG_p),
                 .MYTMG_p               (MYTMG_p),
                 .PAGEO_p               (PAGEO_p),
                 .PAMEO_p               (PAMEO_p),
                 // Inputs
                 .MAI02_p               (MAI02_p),
                 .MAI03_p               (MAI03_p),
                 .MAI04_p               (MAI04_p),
                 .MAI05_p               (MAI05_p),
                 .MAI06_p               (MAI06_p),
                 .MAI07_p               (MAI07_p),
                 .MAI08_p               (MAI08_p),
                 .MAI09_p               (MAI09_p),
                 .MAI10_p               (MAI10_p),
                 .MAI11_p               (MAI11_p),
                 .MAI12_p               (MAI12_p),
                 .MAI13_p               (MAI13_p),
                 .MAI14_p               (MAI14_p),
                 .MAI15_p               (MAI15_p),
                 .MAI16_p               (MAI16_p),
                 .MDI01_p               (MDI01_p),
                 .MDI02_p               (MDI02_p),
                 .MDI03_p               (MDI03_p),
                 .MDI04_p               (MDI04_p),
                 .MDI05_p               (MDI05_p),
                 .MDI06_p               (MDI06_p),
                 .MDI07_p               (MDI07_p),
                 .MDI08_p               (MDI08_p),
                 .MDI09_p               (MDI09_p),
                 .MDI10_p               (MDI10_p),
                 .MDI11_p               (MDI11_p),
                 .MDI12_p               (MDI12_p),
                 .MDI13_p               (MDI13_p),
                 .MDI14_p               (MDI14_p),
                 .MDI15_p               (MDI15_p),
                 .MDI16_p               (MDI16_p),
                 .MEMCI_p               (MEMCI_p),
                 .MREAD_p               (MREAD_p),
                 .MSTCL_n               (MSTCL_n),
                 .PBNKA_p               (PBNKA_p),
                 .PBNKB_p               (PBNKB_p),
                 .PBNKC_p               (PBNKC_p),
                 .PBNKD_p               (PBNKD_p));
   
   h316_mlo mlo (// Outputs
                 .OTB01_p               (OTBxx_p[1]),
                 .OTB02_p               (OTBxx_p[2]),
                 .OTB03_p               (OTBxx_p[3]),
                 .OTB04_p               (OTBxx_p[4]),
                 .OTB05_p               (OTBxx_p[5]),
                 .OTB06_p               (OTBxx_p[6]),
                 .OTB07_p               (OTBxx_p[7]),
                 .OTB08_p               (OTBxx_p[8]),
                 .OTB09_p               (OTBxx_p[9]),
                 .OTB10_p               (OTBxx_p[10]),
                 .OTB11_p               (OTBxx_p[11]),
                 .OTB12_p               (OTBxx_p[12]),
                 .OTB13_p               (OTBxx_p[13]),
                 .OTB14_p               (OTBxx_p[14]),
                 .OTB15_p               (OTBxx_p[15]),
                 .OTB16_p               (OTBxx_p[16]),
                 // Inputs
                 .ADB07_n               (ADBxx_n[7]),
                 .ADB08_n               (ADBxx_n[8]),
                 .ADB09_n               (ADBxx_n[9]),
                 .ADB10_n               (ADBxx_n[10]), 
                 /*AUTOINST*/
                 // Outputs
                 .PROTX_p               (PROTX_p),
                 // Inputs
                 .BSD02_p               (BSD02_p),
                 .CMKXX_n               (CMKXX_n),
                 .D03FF_n               (D03FF_n),
                 .D04FF_n               (D04FF_n),
                 .D05FF_n               (D05FF_n),
                 .D06FF_n               (D06FF_n),
                 .D07FF_n               (D07FF_n),
                 .MSTCL_n               (MSTCL_n),
                 .SMK20_n               (SMK20_n));

   h316_sdmc dmc (// Outputs
                  .DAL01_n              (DALxx_n[1]),
                  .DAL02_n              (DALxx_n[2]),
                  .DAL03_n              (DALxx_n[3]),
                  .DAL04_n              (DALxx_n[4]),
                  .DAL05_n              (DALxx_n[5]),
                  .DAL06_n              (DALxx_n[6]),
                  .DAL07_n              (DALxx_n[7]),
                  .DAL08_n              (DALxx_n[8]),
                  .DAL09_n              (DALxx_n[9]),
                  .DAL10_n              (DALxx_n[10]),
                  .DAL11_n              (DALxx_n[11]),
                  .DAL12_n              (DALxx_n[12]),
                  .DAL13_n              (DALxx_n[13]),
                  .DAL14_n              (DALxx_n[14]),
                  .DAL15_n              (DALxx_n[15]),
                  .DAL16_n              (DALxx_n[16]),
                  .OTB01_p              (OTBxx_p[1]),
                  .OTB02_p              (OTBxx_p[2]),
                  .OTB03_p              (OTBxx_p[3]),
                  .OTB04_p              (OTBxx_p[4]),
                  .OTB05_p              (OTBxx_p[5]),
                  .OTB06_p              (OTBxx_p[6]),
                  .OTB07_p              (OTBxx_p[7]),
                  .OTB08_p              (OTBxx_p[8]),
                  .OTB09_p              (OTBxx_p[9]),
                  .OTB10_p              (OTBxx_p[10]),
                  .OTB11_p              (OTBxx_p[11]),
                  .OTB12_p              (OTBxx_p[12]),
                  .OTB13_p              (OTBxx_p[13]),
                  .OTB14_p              (OTBxx_p[14]),
                  .OTB15_p              (OTBxx_p[15]),
                  .OTB16_p              (OTBxx_p[16]),
                  // Inputs
                  .DIL01_p              (DILxx_p[1]),
                  .DIL02_p              (DILxx_p[2]),
                  .DIL03_p              (DILxx_p[3]),
                  .DIL04_p              (DILxx_p[4]),
                  .DIL05_p              (DILxx_p[5]),
                  .DIL06_p              (DILxx_p[6]),
                  .DIL07_p              (DILxx_p[7]),
                  .DIL08_p              (DILxx_p[8]),
                  .DIL09_p              (DILxx_p[9]),
                  .DIL10_p              (DILxx_p[10]),
                  .DIL11_p              (DILxx_p[11]),
                  .DIL12_p              (DILxx_p[12]),
                  .DIL13_p              (DILxx_p[13]),
                  .DIL14_p              (DILxx_p[14]),
                  .DIL15_p              (DILxx_p[15]),
                  .DIL16_p              (DILxx_p[16]),
                  .INB01_n              (INBxx_n[1]),
                  .INB02_n              (INBxx_n[2]),
                  .INB03_n              (INBxx_n[3]),
                  .INB04_n              (INBxx_n[4]),
                  .INB05_n              (INBxx_n[5]),
                  .INB06_n              (INBxx_n[6]),
                  .INB07_n              (INBxx_n[7]),
                  .INB08_n              (INBxx_n[8]),
                  .INB09_n              (INBxx_n[9]),
                  .INB10_n              (INBxx_n[10]),
                  .INB11_n              (INBxx_n[11]),
                  .INB12_n              (INBxx_n[12]),
                  .INB13_n              (INBxx_n[13]),
                  .INB14_n              (INBxx_n[14]),
                  .INB15_n              (INBxx_n[15]),
                  .INB16_n              (INBxx_n[16]),
                  .M16FF_p              (MxxFF_p[16]),
                  /*AUTOINST*/
                  // Outputs
                  .CLMTR_n              (CLMTR_n),
                  .DCY2X_p              (DCY2X_p),
                  .DMCCY_n              (DMCCY_n),
                  .DMCWR_n              (DMCWR_n),
                  .DMCYQ_n              (DMCYQ_n),
                  .DMRRL_p              (DMRRL_p),
                  .EIYTS_n_A            (EIYTS_n_A),
                  .ERLXX_n              (ERLXX_n),
                  .GENOP_n              (GENOP_n),
                  .IOACY_p              (IOACY_p),
                  .MADCL_n_A            (MADCL_n_A),
                  .MADCL_n_C            (MADCL_n_C),
                  .NRMOP_p_D            (NRMOP_p_D),
                  .OBINH_n              (OBINH_n),
                  .RRCXX_n              (RRCXX_n),
                  .RTCCL_n              (RTCCL_n),
                  .RTCEN_n_A            (RTCEN_n_A),
                  .SETTA_n              (SETTA_n),
                  .T2DMC_p              (T2DMC_p),
                  .T2DMC_n              (T2DMC_n),
                  .TLBFF_n              (TLBFF_n),
                  .Y02FF_n              (Y02FF_n),
                  .Y03FF_n              (Y03FF_n),
                  .Y04FF_n              (Y04FF_n),
                  .Y05FF_n              (Y05FF_n),
                  .Y06FF_n              (Y06FF_n),
                  .Y07FF_n              (Y07FF_n),
                  .Y08FF_n              (Y08FF_n),
                  // Inouts
                  .IYB09_p              (IYB09_p),
                  .IYB10_p              (IYB10_p),
                  .IYB11_p              (IYB11_p),
                  .IYB12_p              (IYB12_p),
                  .IYB13_p              (IYB13_p),
                  .IYB14_p              (IYB14_p),
                  .IYB15_p              (IYB15_p),
                  .IYB16_p              (IYB16_p),
                  .M01FF_n              (M01FF_n),
                  .M02FF_n              (M02FF_n),
                  .M03FF_n              (M03FF_n),
                  .M04FF_n              (M04FF_n),
                  .M05FF_n              (M05FF_n),
                  .M06FF_n              (M06FF_n),
                  .M07FF_n              (M07FF_n),
                  .M08FF_n              (M08FF_n),
                  .M09FF_n              (M09FF_n),
                  .M10FF_n              (M10FF_n),
                  .M11FF_n              (M11FF_n),
                  .M12FF_n              (M12FF_n),
                  .M13FF_n              (M13FF_n),
                  .M14FF_n              (M14FF_n),
                  .M15FF_n              (M15FF_n),
                  .M16FF_n              (M16FF_n),
                  .RPTT2_p              (RPTT2_p),
                  // Inputs
                  .ACYLF_p              (ACYLF_p),
                  .ACYLF_n              (ACYLF_n),
                  .BREAK_n              (BREAK_n),
                  .DIVOP_n              (DIVOP_n),
                  .EIYHS_p              (EIYHS_p),
                  .EOINS_n              (EOINS_n),
                  .ICYLF_p              (ICYLF_p),
                  .IG063_n              (IG063_n),
                  .IOGRP_n              (IOGRP_n),
                  .IRSOP_p              (IRSOP_p),
                  .MCRST_p              (MCRST_p),
                  .MCSET_p              (MCSET_p),
                  .MCTLG_p              (MCTLG_p),
                  .MPYOP_n              (MPYOP_n),
                  .MSTCL_n              (MSTCL_n),
                  .NRMOP_p              (NRMOP_p),
                  .RRCXX_p              (RRCXX_p),
                  .RRLIN_n              (RRLIN_n),
                  .SCZR0_p              (SCZR0_p),
                  .TL1FF_p_A            (TL1FF_p_A),
                  .TL23F_n              (TL23F_n),
                  .TL24F_p              (TL24F_p),
                  .TL2FF_p_B            (TL2FF_p_B),
                  .TL3FF_p_A            (TL3FF_p_A),
                  .TL3FF_n              (TL3FF_n),
                  .TL4FF_p_A            (TL4FF_p_A),
                  .WRITE_n              (WRITE_n));

endmodule // h316_mfm_bus

// Local Variables:
// verilog-library-directories:("." "../net")
// End:
