/***********************************************************************
 *
 * h316 - Gate-level H316 processor testbench
 * 
 * Copyright (C) 2007, 2011, 2017 Adrian Wise
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of version 3 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See http://www.series16.adrianwise.co.uk
 *   mailto:apw@adrianwise.co.uk
 * 
 ***********************************************************************
 * 
 * This is a simple testbench running the gate-level h316 CPU.
 * 
 **********************************************************************/

module h316_tb ();

   localparam BAUD = 153600;
   localparam NDMC = 16;        // Number of channel exercisers

   // These are inputs to h316_cpu_bus (mostly from missing front-panel)
   wire        MSTCL_n;                // To DUT of h316_cpu_bus.v
   reg         CLAMP_n;                // To DUT of h316_cpu_bus.v
   reg         CLBMP_n;                // To DUT of h316_cpu_bus.v
   reg         CLMMP_n;                // To DUT of h316_cpu_bus.v
   reg         CLPMP_n;                // To DUT of h316_cpu_bus.v
   reg         DSPLA_p;                // To DUT of h316_cpu_bus.v
   reg         DSPLM_p;                // To DUT of h316_cpu_bus.v
   reg         DSPLO_p;                // To DUT of h316_cpu_bus.v
   reg         DSPLB_p;                // To DUT of h316_cpu_bus.v
   reg         DSPLY_p;                // To DUT of h316_cpu_bus.v
   reg         ENTRA_n;                // To DUT of h316_cpu_bus.v
   reg         ENTRB_n;                // To DUT of h316_cpu_bus.v
   reg         ENTRM_n;                // To DUT of h316_cpu_bus.v
   reg         ENTRP_n;                // To DUT of h316_cpu_bus.v
   reg         FETCH_n;                // To DUT of h316_cpu_bus.v
   reg         MASTO_n;                // To DUT of h316_cpu_bus.v
   reg         MEMAC_n;                // To DUT of h316_cpu_bus.v
   reg         PARCK_n;                // To DUT of h316_cpu_bus.v
   reg         PFINT_n;                // To DUT of h316_cpu_bus.v
   reg         READY_n;                // To DUT of h316_cpu_bus.v
   reg         RUNMD_n;                // To DUT of h316_cpu_bus.v
   reg  [1:16] SDRxx_n;                // To DUT of h316_cpu_bus.v
   reg         START_p;                // To DUT of h316_cpu_bus.v
   reg         STEPP_n;                // To DUT of h316_cpu_bus.v
   reg   [1:4] sense;

   // Declare 16-bit bus (if AUTO selects ASR, only get an 8-bit bus)
   wire [1:16] INBxx_n;                // To DUT of h316_cpu_bus.v

   // These wires are inouts to/from h316_cpu_bus.v
   wire [1:4]  SENSx_n;                // To DUT of h316_cpu_bus.v
   wire [1:4]  SENSx_p;                // To DUT of h316_cpu_bus.v

   /*AUTOWIRE*/
   // Beginning of automatic wires (for undeclared instantiated-module outputs)
   wire [7:16]          ADBxx_n;                // From DUT of h316_mfm_bus.v
   wire                 BANKA_n;                // From DUT of h316_mfm_bus.v
   wire                 BANKB_n;                // From DUT of h316_mfm_bus.v
   wire                 BANKC_n;                // From DUT of h316_mfm_bus.v
   wire                 BANKD_n;                // From DUT of h316_mfm_bus.v
   wire                 CMKXX_n;                // From DUT of h316_mfm_bus.v
   wire [1:16]          DALxx_n;                // From DUT of h316_mfm_bus.v
   wire [1:16]          DILxx_p;                // From vdmc of h316_vdmc.v
   wire                 DRLIN_n;                // From ASR of h316_asr_bus.v, ...
   wire                 ENDOP_n;                // From DUT of h316_mfm_bus.v
   wire                 ERLXX_n;                // From DUT of h316_mfm_bus.v
   wire [1:16]          INDxx_n;                // From DUT of h316_mfm_bus.v
   wire                 LMPRN_n;                // From DUT of h316_mfm_bus.v
   wire                 MADCL_n_A;              // From DUT of h316_mfm_bus.v
   wire                 MADCL_n_C;              // From DUT of h316_mfm_bus.v
   wire [5:16]          MADxx_n;                // From DUT of h316_mfm_bus.v
   wire                 MARRC_n_A;              // From DUT of h316_mfm_bus.v
   wire                 MARRC_n_B;              // From DUT of h316_mfm_bus.v
   wire                 MBSYX_p;                // From DUT of h316_mfm_bus.v
   wire                 MBUSY_p;                // From DUT of h316_mfm_bus.v
   wire [1:16]          MDOxx_p;                // From DUT of h316_mfm_bus.v
   wire [1:16]          MMDxx_n;                // From MEM of h316_memory.v
   wire [1:16]          MMxxF_n;                // From MEM of h316_memory.v
   wire                 MSTRB_n;                // From DUT of h316_mfm_bus.v
   wire                 MXTMG_p;                // From DUT of h316_mfm_bus.v
   wire                 MYTMG_p;                // From DUT of h316_mfm_bus.v
   wire [1:16]          MxxFF_p;                // From DUT of h316_mfm_bus.v
   wire                 OCPLS_n;                // From DUT of h316_mfm_bus.v
   wire [1:16]          OTBxx_p;                // From DUT of h316_mfm_bus.v
   wire                 PIL00_n;                // From ASR of h316_asr_bus.v, ...
   wire                 PILVD_n;                // From vdmc of h316_vdmc.v
   wire                 RRLIN_n;                // From DUT of h316_mfm_bus.v
   wire                 SMK01_n;                // From DUT of h316_mfm_bus.v
   wire                 SMKXX_n;                // From DUT of h316_mfm_bus.v
   wire                 STROB_n;                // From DUT of h316_mfm_bus.v
   wire                 TYSIG_n;                // To/From ASR of h316_asr_bus.v
   wire                 WRITE_n;                // From DUT of h316_mfm_bus.v
   wire                 XTIMG_p;                // From DUT of h316_mfm_bus.v
   wire                 YTIMG_p;                // From DUT of h316_mfm_bus.v
   wire [1:16]          exit_code;              // From vsim of h316_vsim.v
   wire                 exit_sim;               // From vsim of h316_vsim.v
   wire                 xBANKA_n;               // From DUT of h316_mfm_bus.v
   wire                 xBANKB_n;               // From DUT of h316_mfm_bus.v
   wire                 xBANKC_n;               // From DUT of h316_mfm_bus.v
   wire                 xBANKD_n;               // From DUT of h316_mfm_bus.v
   wire [5:16]          xMADxx_n;               // From DUT of h316_mfm_bus.v
   wire                 xWRITE_n;               // From DUT of h316_mfm_bus.v
   // End of automatics

   wire               sample_state;

   reg                MSTCL_p;
   reg                clk = 1'b0;
   reg                reset_N = 1'b0;

   integer            clk_count        = 0;
   integer            clock_limit      = 0;
   integer            clock_limit_exit = 3;
   integer            done_pc          = 0;
   integer            done_pc_count    = 1;
   integer            done_pc_counter  = 0;
   integer            dump_start       = 0;
   reg [256*8-1:1]    wavefilename;

   genvar i;

   ccc_nand1 u_MSTCL_n (.op(MSTCL_n), .in1(MSTCL_p));
   
   h316_mfm_bus DUT (/*AUTOINST*/
                     // Outputs
                     .LMPRN_n           (LMPRN_n),
                     .INDxx_n           (INDxx_n[1:16]),
                     .OTBxx_p           (OTBxx_p[1:16]),
                     .ADBxx_n           (ADBxx_n[7:16]),
                     .SMKXX_n           (SMKXX_n),
                     .OCPLS_n           (OCPLS_n),
                     .SMK01_n           (SMK01_n),
                     .ERLXX_n           (ERLXX_n),
                     .CMKXX_n           (CMKXX_n),
                     .ENDOP_n           (ENDOP_n),
                     .RRLIN_n           (RRLIN_n),
                     .MxxFF_p           (MxxFF_p[1:16]),
                     .MADxx_n           (MADxx_n[5:16]),
                     .BANKD_n           (BANKD_n),
                     .BANKC_n           (BANKC_n),
                     .BANKB_n           (BANKB_n),
                     .WRITE_n           (WRITE_n),
                     .MBSYX_p           (MBSYX_p),
                     .BANKA_n           (BANKA_n),
                     .MADCL_n_A         (MADCL_n_A),
                     .STROB_n           (STROB_n),
                     .MADCL_n_C         (MADCL_n_C),
                     .XTIMG_p           (XTIMG_p),
                     .YTIMG_p           (YTIMG_p),
                     .MDOxx_p           (MDOxx_p[1:16]),
                     .xMADxx_n          (xMADxx_n[5:16]),
                     .xBANKD_n          (xBANKD_n),
                     .xBANKC_n          (xBANKC_n),
                     .xBANKB_n          (xBANKB_n),
                     .xWRITE_n          (xWRITE_n),
                     .MBUSY_p           (MBUSY_p),
                     .xBANKA_n          (xBANKA_n),
                     .MARRC_n_A         (MARRC_n_A),
                     .MSTRB_n           (MSTRB_n),
                     .MARRC_n_B         (MARRC_n_B),
                     .MXTMG_p           (MXTMG_p),
                     .MYTMG_p           (MYTMG_p),
                     .DALxx_n           (DALxx_n[1:16]),
                     // Inouts
                     .SENSx_p           (SENSx_p[1:4]),
                     .SENSx_n           (SENSx_n[1:4]),
                     .DRLIN_n           (DRLIN_n),
                     .PIL00_n           (PIL00_n),
                     // Inputs
                     .PFINT_n           (PFINT_n),
                     .MSTCL_n           (MSTCL_n),
                     .ENTRA_n           (ENTRA_n),
                     .ENTRM_n           (ENTRM_n),
                     .FETCH_n           (FETCH_n),
                     .RUNMD_n           (RUNMD_n),
                     .CLBMP_n           (CLBMP_n),
                     .CLPMP_n           (CLPMP_n),
                     .START_p           (START_p),
                     .DSPLO_p           (DSPLO_p),
                     .DSPLY_p           (DSPLY_p),
                     .ENTRB_n           (ENTRB_n),
                     .ENTRP_n           (ENTRP_n),
                     .MEMAC_n           (MEMAC_n),
                     .STEPP_n           (STEPP_n),
                     .READY_n           (READY_n),
                     .CLAMP_n           (CLAMP_n),
                     .CLMMP_n           (CLMMP_n),
                     .MASTO_n           (MASTO_n),
                     .DSPLM_p           (DSPLM_p),
                     .DSPLA_p           (DSPLA_p),
                     .DSPLB_p           (DSPLB_p),
                     .SDRxx_n           (SDRxx_n[1:16]),
                     .INBxx_n           (INBxx_n[1:16]),
                     .PARCK_n           (PARCK_n),
                     .MMxxF_n           (MMxxF_n[1:16]),
                     .MMDxx_n           (MMDxx_n[1:16]),
                     .DILxx_p           (DILxx_p[1:16]));
  
   h316_asr_bus ASR (/*AUTOINST*/
                     // Outputs
                     .INBxx_n           (INBxx_n[9:16]),
                     .DRLIN_n           (DRLIN_n),
                     .PIL00_n           (PIL00_n),
                     // Inouts
                     .TYSIG_n           (TYSIG_n),
                     // Inputs
                     .ADBxx_n           (ADBxx_n[7:16]),
                     .OTBxx_p           (OTBxx_p[9:16]),
                     .CMKXX_n           (CMKXX_n),
                     .MSTCL_n           (MSTCL_n),
                     .OCPLS_n           (OCPLS_n),
                     .RRLIN_n           (RRLIN_n),
                     .SMK01_n           (SMK01_n),
                     .SMKXX_n           (SMKXX_n));
     
   //
   // The ASR interface's INPBL+ latch isn't reset
   // and this causes an 'X' to be stored in TYCFB when
   // an OCP '104 is executed to put the interface into
   // output mode.
   // Pick one of these to reset it one way or the other (it
   // doesn't matter which - it's the 'X' that's the problem).
   //
   buf (strong0, pull1) #16 u_INPBL_n (ASR.asr.INPBL_n, MSTCL_n);
   //buf (strong0, pull1) #16 u_INPBL_p (ASR.asr.INPBL_p, MSTCL_n);

   h316_memory MEM (/*AUTOINST*/
                    // Outputs
                    .MMxxF_n            (MMxxF_n[1:16]),
                    .MMDxx_n            (MMDxx_n[1:16]),
                    // Inputs
                    .MBSYX_p            (MBSYX_p),
                    .XTIMG_p            (XTIMG_p),
                    .YTIMG_p            (YTIMG_p),
                    .WRITE_n            (WRITE_n),
                    .STROB_n            (STROB_n),
                    .MADCL_n_A          (MADCL_n_A),
                    .MADCL_n_C          (MADCL_n_C),
                    .BANKA_n            (BANKA_n),
                    .BANKB_n            (BANKB_n),
                    .BANKC_n            (BANKC_n),
                    .BANKD_n            (BANKD_n),
                    .MADxx_n            (MADxx_n[5:16]),
                    .MxxFF_p            (MxxFF_p[1:16]),
                    .MBUSY_p            (MBUSY_p),
                    .MXTMG_p            (MXTMG_p),
                    .MYTMG_p            (MYTMG_p),
                    .xWRITE_n           (xWRITE_n),
                    .MSTRB_n            (MSTRB_n),
                    .MARRC_n_A          (MARRC_n_A),
                    .MARRC_n_B          (MARRC_n_B),
                    .xBANKA_n           (xBANKA_n),
                    .xBANKB_n           (xBANKB_n),
                    .xBANKC_n           (xBANKC_n),
                    .xBANKD_n           (xBANKD_n),
                    .xMADxx_n           (xMADxx_n[5:16]),
                    .MDOxx_p            (MDOxx_p[1:16]));

   
   defparam ASR.asr.u_TYCLK_p.BAUD = BAUD;
     
   sim_asr #(.BAUD(BAUD)) i_sim_asr (.TYSIG_n(TYSIG_n), .MCSET_p(MCSET_p));

   h316_vdmc
     #(.NDMC                            (NDMC))
   vdmc
     (/*AUTOINST*/
      // Outputs
      .INBxx_n                          (INBxx_n[1:16]),
      .DRLIN_n                          (DRLIN_n),
      .PIL00_n                          (PIL00_n),
      .PILVD_n                          (PILVD_n),
      .DILxx_p                          (DILxx_p[1:16]),
      // Inputs
      .clk                              (clk),
      .reset_N                          (reset_N),
      .MSTCL_n                          (MSTCL_n),
      .ADBxx_n                          (ADBxx_n[7:16]),
      .OTBxx_p                          (OTBxx_p[1:16]),
      .OCPLS_n                          (OCPLS_n),
      .SMK01_n                          (SMK01_n),
      .RRLIN_n                          (RRLIN_n),
      .ERLXX_n                          (ERLXX_n),
      .DALxx_n                          (DALxx_n[1:16]));

   h316_vsim vsim
     (/*AUTOINST*/
      // Outputs
      .DRLIN_n                          (DRLIN_n),
      .exit_sim                         (exit_sim),
      .exit_code                        (exit_code[1:16]),
      // Inputs
      .MSTCL_n                          (MSTCL_n),
      .ADBxx_n                          (ADBxx_n[7:16]),
      .OTBxx_p                          (OTBxx_p[1:16]),
      .RRLIN_n                          (RRLIN_n));

   // Pull-ups on I/O bus input lines
   pullup pu_DRLIN_n  (DRLIN_n);
   pullup pu_PIL00_n  (PIL00_n);
   
   generate
      for (i=1; i<16; i=i+1) begin
         pullup pu_INBxx_n (INBxx_n[i]);
      end
      for (i=1; i<4; i=i+1) begin
         nand (strong0, pull1) #16 u_SENSx_p (SENSx_p[i], ~sense[i]);
         nand (strong0, pull1) #16 u_SENSx_n (SENSx_n[i],  sense[i]);
      end
   endgenerate
   
   /*
    * Pick up buses from the gate-level implementation for
    * use/display in the testbench.
    */
   wire [1:16] PxxFF_p;
   wire [1:16] AxxFF_p;
   wire [1:16] BxxFF_p;
   wire        CBITF_p;
   wire [1:16] XxxFF_p;
   
   wire [1:16] YxxFF_p;

   wire [1:16] DxxFF_p;
   wire [1:16] ExxFF_p;

   wire        MCSET_p;
   wire        RUNFF_p;
   wire        TL1FF_p;
   wire        FCYEF_p;
   wire        FCYLF_p;

   wire        EMXTS_p;
   reg         XxxFF_valid;
   reg         checking;
   
   assign sample_state = DUT.cpu.EOINS_p & DUT.cpu.TL4FF_p;
   
   assign PxxFF_p = {DUT.cpu.P01FF_p, DUT.cpu.P02FF_p, DUT.cpu.P03FF_p, DUT.cpu.P04FF_p,
                     DUT.cpu.P05FF_p, DUT.cpu.P06FF_p, DUT.cpu.P07FF_p, DUT.cpu.P08FF_p,
                     DUT.cpu.P09FF_p, DUT.cpu.P10FF_p, DUT.cpu.P11FF_p, DUT.cpu.P12FF_p,
                     DUT.cpu.P13FF_p, DUT.cpu.P14FF_p, DUT.cpu.P15FF_p, DUT.cpu.P16FF_p};
   assign AxxFF_p = {DUT.cpu.A01FF_p, DUT.cpu.A02FF_p, DUT.cpu.A03FF_p, DUT.cpu.A04FF_p,
                     DUT.cpu.A05FF_p, DUT.cpu.A06FF_p, DUT.cpu.A07FF_p, DUT.cpu.A08FF_p,
                     DUT.cpu.A09FF_p, DUT.cpu.A10FF_p, DUT.cpu.A11FF_p, DUT.cpu.A12FF_p,
                     DUT.cpu.A13FF_p, DUT.cpu.A14FF_p, DUT.cpu.A15FF_p, DUT.cpu.A16FF_p};
   assign BxxFF_p = {DUT.cpu.B01FF_p, DUT.cpu.B02FF_p, DUT.cpu.B03FF_p, DUT.cpu.B04FF_p,
                     DUT.cpu.B05FF_p, DUT.cpu.B06FF_p, DUT.cpu.B07FF_p, DUT.cpu.B08FF_p,
                     DUT.cpu.B09FF_p, DUT.cpu.B10FF_p, DUT.cpu.B11FF_p, DUT.cpu.B12FF_p,
                     DUT.cpu.B13FF_p, DUT.cpu.B14FF_p, DUT.cpu.B15FF_p, DUT.cpu.B16FF_p};
   assign CBITF_p = DUT.cpu.CBITF_p;
   assign XxxFF_p = {DUT.cpu.X01FF_p, DUT.cpu.X02FF_p, DUT.cpu.X03FF_p, DUT.cpu.X04FF_p,
                     DUT.cpu.X05FF_p, DUT.cpu.X06FF_p, DUT.cpu.X07FF_p, DUT.cpu.X08FF_p,
                     DUT.cpu.X09FF_p, DUT.cpu.X10FF_p, DUT.cpu.X11FF_p, DUT.cpu.X12FF_p,
                     DUT.cpu.X13FF_p, DUT.cpu.X14FF_p, DUT.cpu.X15FF_p, DUT.cpu.X16FF_p};
   assign YxxFF_p = {DUT.cpu.Y01FF_p, DUT.cpu.Y02FF_p, DUT.cpu.Y03FF_p, DUT.cpu.Y04FF_p,
                     DUT.cpu.Y05FF_p, DUT.cpu.Y06FF_p, DUT.cpu.Y07FF_p, DUT.cpu.Y08FF_p,
                     DUT.cpu.Y09FF_p, DUT.cpu.Y10FF_p, DUT.cpu.Y11FF_p, DUT.cpu.Y12FF_p,
                     DUT.cpu.Y13FF_p, DUT.cpu.Y14FF_p, DUT.cpu.Y15FF_p, DUT.cpu.Y16FF_p};
   assign DxxFF_p = {DUT.cpu.D01FF_p, DUT.cpu.D02FF_p, DUT.cpu.D03FF_p, DUT.cpu.D04FF_p,
                     DUT.cpu.D05FF_p, DUT.cpu.D06FF_p, DUT.cpu.D07FF_p, DUT.cpu.D08FF_p,
                     DUT.cpu.D09FF_p, DUT.cpu.D10FF_p, DUT.cpu.D11FF_p, DUT.cpu.D12FF_p,
                     DUT.cpu.D13FF_p, DUT.cpu.D14FF_p, DUT.cpu.D15FF_p, DUT.cpu.D16FF_p};
   assign ExxFF_p = {DUT.cpu.E01FF_p, DUT.cpu.E02FF_p, DUT.cpu.E03FF_p, DUT.cpu.E04FF_p,
                     DUT.cpu.E05FF_p, DUT.cpu.E06FF_p, DUT.cpu.E07FF_p, DUT.cpu.E08FF_p,
                     DUT.cpu.E09FF_p, DUT.cpu.E10FF_p, DUT.cpu.E11FF_p, DUT.cpu.E12FF_p,
                     DUT.cpu.E13FF_p, DUT.cpu.E14FF_p, DUT.cpu.E15FF_p, DUT.cpu.E16FF_p};

   assign MCSET_p = DUT.MCSET_p;
   assign RUNFF_p = DUT.cpu.RUNFF_p;
   assign TL1FF_p = DUT.cpu.TL1FF_p;
   assign FCYEF_p = DUT.cpu.FCYEF_p;
   assign FCYLF_p = DUT.cpu.FCYLF_p;
   

   assign EMXTS_p = DUT.cpu.EMXTS_p;

   // X register is invalid (contains X) until first write to it
   initial begin : Xvalid_p
      
      XxxFF_valid = 0;

      forever begin
         @(posedge EMXTS_p or negedge MSTCL_p) begin
            if (EMXTS_p)
              XxxFF_valid = 1;
            else /* including X! */
              XxxFF_valid = 0;
         end
      end
   end // block: Xvalid_p

   // Produce a 'clk' as used with modern positive-edge synchronous flip-flop logic
   //
   // Each clock is two phases of the H316's latch-based design
   //
   always @(posedge MCSET_p) begin : clk_p
      clk <= ~clk;
      if (!reset_N)
        reset_N <= #10 1'b1;
   end

   // Limit simulation based on number of clocks
   initial begin : clk_limit_p
      if ($value$plusargs("CLOCK_LIMIT=%d", clock_limit)) begin
         $display("Clock limit set to %d", clock_limit);
      end
      if ($value$plusargs("CLOCK_LIMIT_EXIT=%d", clock_limit_exit)) begin
         $display("Clock limit exit code set to %d", clock_limit_exit);
      end
      if ($value$plusargs("DONE_PC=%o", done_pc)) begin
         $display("Done PC set to %o", done_pc);
      end
      if ($value$plusargs("DONE_PC_COUNT=%d", done_pc_count)) begin
         $display("Done PC count set to %d", done_pc_count);
      end
      
      forever begin
         @(posedge clk);   

         if ((clock_limit != 0) &&
           (clk_count >= clock_limit)) begin
            i_sim_asr.flush;
            $display("Clock limit reached - ending simulation");
            $finish_and_return(clock_limit_exit);
         end
         
         if ((clk_count % 1000000) == 0)
           $display("%t: p=%o, clk_count = %d", $time, PxxFF_p, clk_count);
         
         clk_count = clk_count + 1;
      end // forever begin
   end // block: clk_limit_p

   // Monitor for exit conditions
   always @(negedge RUNFF_p) begin : runff_p

      // Note that 'clk' stops once RUNFF goes low
      if (MSTCL_n) begin
         $display("%t RUNFF_p low p=%o, clk_count=%d", $time, PxxFF_p, clk_count);
         #1000;
         i_sim_asr.flush;
         $display("RUNFF_p low - ending simulation");
         $finish_and_return(2);
      end
      
   end // block: runff_p
   
   always @(posedge clk) begin : exit_p

      if ((reset_N) && (exit_sim)) begin
         $display("%t Exit Code=%o, p=%o, clk_count=%d", $time, exit_code, PxxFF_p, clk_count);
         $finish_and_return(exit_code);
      end

      if (TL1FF_p && FCYEF_p && FCYLF_p) begin
         if ((done_pc != 0) && (done_pc == PxxFF_p[2:16])) begin
            done_pc_counter = done_pc_counter + 1;
            if (done_pc_counter >= done_pc_count) begin
               i_sim_asr.flush;
               $display("%t Done PC (%o) reached for final time - ending simulation",
                        $time, done_pc);
               $finish_and_return(0);
            end else begin
               $display("%t Done PC (%o) reached (counter = %d of %d)",
                        $time, done_pc, done_pc_counter, done_pc_count);
            end
         end // if ((done_pc != 0) && (done_pc == PxxFF_p[2:16]))
      end // if (TL1FF_p && FCYEF_p && FCYLF_p)
   end // block: exit_p
   
   initial begin : frontpanel_p
      #0;
      sense <= 4'd0;
      checking <= 1'b0;
      
      CLAMP_n <= 1'b1;
      CLBMP_n <= 1'b1;
      CLMMP_n <= 1'b1;
      CLPMP_n <= 1'b1;
      DSPLA_p <= 1'b1;
      DSPLM_p <= 1'b1;
      DSPLO_p <= 1'b1;
      DSPLB_p <= 1'b1;
      DSPLY_p <= 1'b1;
      ENTRA_n <= 1'b1;
      ENTRB_n <= 1'b1;
      ENTRM_n <= 1'b1;
      ENTRP_n <= 1'b1;
      FETCH_n <= 1'b1;
      MASTO_n <= 1'b1;
      MEMAC_n <= 1'b1;
      PARCK_n <= 1'b1;
      PFINT_n <= 1'b1;
      RUNMD_n <= 1'b1;
      SDRxx_n[1:16] <= 16'hFFFF;
      STEPP_n <= 1'b1;
      START_p <= 1'b0;
      READY_n <= 1'b1;

      MSTCL_p <= 1'b1;
      CLAMP_n <= 1'b0;
      CLBMP_n <= 1'b0;
      CLMMP_n <= 1'b0;
      CLPMP_n <= 1'b0;
      #1000;
         
      MSTCL_p <= 1'b0;
      CLAMP_n <= 1'b1;
      CLBMP_n <= 1'b1;
      CLMMP_n <= 1'b1;
      CLPMP_n <= 1'b1;
      
      #1000;

      checking <= 1'b1;
      
      RUNMD_n <= 1'b0;
      START_p <= 1'b1;
      #1000;
      READY_n <= 1'b0;
      #1000;
      READY_n <= 1'b1;
      #1000;
      START_p <= 1'b0;

   end // block: frontpanel_p
         
   initial begin

      dump_start = 0;
      if ($value$plusargs("DUMP_START=%d", dump_start)) begin
         $display("Dump start set to %d", dump_start);
      end

      if ($value$plusargs("WAVEFILE=%s", wavefilename)) begin
         $dumpfile(wavefilename);

         while (clk_count < dump_start) begin
            @(posedge clk);   
         end

         $dumpvars(1, h316_tb);
         $dumpvars(4, DUT);
         $dumpvars(4, ASR);
         $dumpvars(3, MEM);
         $dumpvars(3, vdmc);
   
      end // if ($value$plusargs("WAVEFILE=%s", wavefilename))
   end // initial begin
   
/* -----\/----- EXCLUDED -----\/-----
   initial begin
      #100000;

      i_sim_asr.send(8'o275);
      #60000;
      i_sim_asr.send(8'o257);
   end
 -----/\----- EXCLUDED -----/\----- */
   
endmodule // h316_tb

// Local Variables:
// verilog-library-directories:("." "../rtl")
// End:
