/*
 * H316 DMC simulation verification component.
 * 
 * Copyright (C) 2015, 2017 Adrian Wise
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of version 3 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See http://www.series16.adrianwise.co.uk
 *   mailto:apw@adrianwise.co.uk
 *
 ***********************************************************************
 * 
 * This models a simple "peripheral" that may be used to
 * enable a verification and test program to return an exit
 * code to terminate the run in a simulation.
 * 
 * OTA '00xx Output A to exit register
 */
module h316_vsim
  #(parameter ADDR = 6'o35) // Device address
   (// Asynchronous CPU I/O-bus interface
    input  wire        MSTCL_n,   // Master clear
    input  wire [7:16] ADBxx_n,   // Address bus
    input  wire [1:16] OTBxx_p,   // Output bus
    input  wire        RRLIN_n,   // Reset ready line (INA, OTA)
    output wire        DRLIN_n,   // Device ready line
    // Interface to test-bench
    output reg         exit_sim = 1'b0, // A write to this peripheral occurred.
    output reg [1:16]  exit_code); // Returned exit code (when exit_sim)
   
   wire [1:6]          device_address;
   wire [1:4]          function_code;
   wire                drlin;
   
   assign function_code  = (~ADBxx_n[ 7:10]);
   assign device_address = (~ADBxx_n[11:16]);
   
   assign drlin = ((device_address == ADDR) & (function_code == 4'd0));

   nand (strong0, pull1) #16 u_DRLIN_n (DRLIN_n, drlin);      

   always @(posedge RRLIN_n or negedge MSTCL_n) begin
      if (!MSTCL_n) begin
         exit_sim  <= 1'b0;
         exit_code <= 16'd0;
      end else if (drlin) begin
         exit_sim  <= 1'b1;
         exit_code <= OTBxx_p;
      end
   end
  
endmodule // h16_vsim

// Local Variables:
// verilog-library-directories:("." "../../rtl")
// eval:(verilog-read-defines)
// eval:(verilog-read-includes)
// End:
