* DMC CONTROLLER TEST PROGRAM
\ABS
*
NDMC\EQU\16\NUMBER OF DMC CHANNELS IN HARDWARE
HSEA\EQU\1\WHETHER HAVE EXTENDED ADDRESSING
*
\IFN\HSEA
\EXD
\ENDC
************************************************************
*
*    I/O DEVICE ADDRESSES FOR VDMC DEVICE
*
CTRL\EQU\'37\CENTRAL DEVICE ADDRESS
CHAN\EQU\'36\CHANNEL DEVICE ADDRESS
*
*    COMMON TO ALL CHANNELS...
*
IDCH\EQU\'1700+CTRL\INA DMC CHANNEL NUMBER
IICH\EQU\'1600+CTRL\READ INTERRUPTING DMC CHANNEL
IECH\EQU\'1500+CTRL\READ ERROR DMC CHANNEL
ODCH\EQU\'0700+CTRL\OTA DMC CHANNEL NUMBER
SKNB\EQU\'0100+CTRL\SKIP IF NOT BUSY
SKNX\EQU\'0300+CTRL\SKIP NO TRANSFER ERRORS
SKNI\EQU\'0400+CTRL\SKIP IF NOT INTERRUPTING
CERR\EQU\'0300+CTRL\CLEAR ALL CHANNELS' ERROR
*
*    PER CHANNEL...
*
ITFS\EQU\'1000+CHAN\INA TRANSFER SIZE
IDRN\EQU\'1200+CHAN\INA DATA PSEUDO-RANDOM NUMBER GENERATOR
ITRN\EQU\'1500+CHAN\INA TIME PSEUDO-RANDOM NUMBER GENERATOR
ITCW\EQU\'1600+CHAN\INA TIME CONTROL WORD
IERR\EQU\'1700+CHAN\INA ERROR BITS
OTFS\EQU\'0000+CHAN\OTA TRANSFER SIZE
ODRN\EQU\'0200+CHAN\OTA DATA PSEUDO-RANDOM NUMBER GENERATOR
OTRN\EQU\'0500+CHAN\OTA TIME PSEUDO-RANDOM NUMBER GENERATOR
OTCW\EQU\'0600+CHAN\OTA TIME CONTROL WORD
CCER\EQU\'0300+CHAN\CLEAR CHANNEL'S ERROR
CCIN\EQU\'0400+CHAN\CLEAR CHANNEL'S INTERRUPT
STIT\EQU\'0500+CHAN\START INPUT TRANSFER
STOT\EQU\'0600+CHAN\START OUTPUT TRANSFER
SCNB\EQU\'0100+CHAN\SKIP IF CHANNEL NOT BUSY
SCNE\EQU\'0300+CHAN\SKIP IF CHANNEL HAS NO ERRORS
SCNI\EQU\'0400+CHAN\SKIP IF CHANNEL NOT INTERRUPTING
\EJCT
************************************************************
*
*    I/O DEVICE ADDRESSES FOR VSIM DEVICE
*
VSIM\EQU\'35\DEVICE ADDRESS
*
EXIT\EQU\VSIM\EXIT CODE
************************************************************
*
*    STANDARD MACHINE ADDRESSES
*
DMC1\EQU\'20\START ADDRESS, DMC CHANNEL 1
*\\\OTHER DMC ADDRESSES ARE COMPUTED
STDI\EQU\'63\STANDARD INTERRUPT LOCATION
*
************************************************************
*
*    STRUCTURE DEFINITIONS
*
*    BUFFER DESCRIPTOR STRUCTURE
*
LFWD\EQU\0\LINKED LIST FORWARD  POINTER
LBWD\EQU\1\LINKED LIST BACKWARD POINTER
BSTR\EQU\2\ACTUAL START ADDRESS OF THIS BUFFER
BSIZ\EQU\3\TOTAL SIZE OF THIS BUFFER
DSTR\EQU\4\DMC START ADDRESS FOR THIS BUFFER
DEND\EQU\5\DMC  END  ADDRESS FOR THIS BUFFER
SEED\EQU\6\RANDOM NUMBER GENERATION SEED
DPRN\EQU\7\DATA PRNG SEED TO TELL VDMC HARDWARE
BSSZ\EQU\8\TOTAL SIZE OF BUFFER DESCRIPTOR
*
*    DMC CHANNEL STRUCTURE
*
LFWD\EQU\0\LINKED LIST FORWARD  POINTER
LBWD\EQU\1\LINKED LIST BACKWARD POINTER
DCBS\EQU\2\BUFFER USED BY THIS CHANNEL
DCSZ\EQU\3\TOTAL SIZE OF CHANNEL STRUCTURE
\EJCT
************************************************************
*
*    SECTOR ZERO VARIABLES
*
\SETB\'100
\ORG\'200
TMP1\BSS\1
TMP2\BSS\1
TMP3\BSS\1
TMP4\BSS\1
FREE\OCT\0\FREE BUFFER LIST
RDYL\OCT\0\LIST OF BUFFERS READY FOR TRANSFERS
FCHL\OCT\0\FREE CHANNEL LIST
CHKL\OCT\0\BUFFER WAITING TO BE CHECKED
CHNN\BSS\1\CHANNEL NUMBER
CSUM\BSS\1\CHECKSUM
*
*    SECTOR ZERO CONSTANTS
*
DMCN\DAC\NDMC\NUMBER OF DMC CHANNELS (BUILD VAR)
MDMC\DAC\-NDMC\-VE NUMBER OF DMC CHANNELS
BGDS\EQU\5\BUFFER GUARD SIZE (IN WORDS)
MBGD\DAC\0-BGDS\-VE SIZE OF BUFFER GUARD
BGRD\DAC\BGDS\+VE SIZE OF BUFFER GUARD
BDSZ\DAC\BSSZ\SIZE OF A BUFFER DESCRIPTOR
DMCA\DAC\DMC1\ADDRESS OF FIRST DMC CONTROL WORD
DSSZ\DAC\DCSZ\SIZE OF DMC CHANNEL STRUCTURE
DCDA\DAC\DCDS\ARRAY OF DMC CHANNEL STRUCTURES
DMCM\OCT\020000\VDMC STANDARD INTERRUPT MASK BIT
DINT\DAC\INTR\INTERRUPT ROUTINE ADDRESS
*
\EJCT
************************************************************
*
*    FIXED SECTOR ZERO LOCATIONS
*
\ORG\'777
TNUM\OCT\0\TEST NUMBER
************************************************************
*
*    START OF TEST PROGRAM
*
STRT\JST\INIT
\JST\TST1
\JST\TST2
*
\CRA\\ZERO EXIT CODE
\STA\TNUM\CLEAR TEST NUMBER
*
\OTA\EXIT
\JMP\*-1
*
\HLT\\DON'T EXPECT TO REACH HERE
\EJCT
************************************************************
*
*    TEST 1 - SIMPLE WRITE AND READ TRANSFERS
*
TST1\DAC\**
\LDA\=1\TEST NUMBER
\STA\TNUM
*
\LDA\='000020\FIXED '20 CYCLES BETWEEN REQUESTS
\JST\ICTC\SET ON ALL CHANNELS
\LDA\=1
\STA\PTST\FLAG TO PTFR - START THE TRANSFER
\STA\PTDR\AND ANOTHER - DIRECTION CONTROL WORD
\CRA\\SELECT CHANNEL 0
\STA\ICDR\NULL TO INTERRUPT CALL-BACK
T101\STA\CHNN\CURRENT CHANNEL NUMBER
T102\LDA\CHNN
\JST\PTFR
\JST\ITRS
*
\LDA\DEND,1
\SUB\DSTR,1
\AOA\\SIZE OF DATA TRANSFER
\MPY\=-5\COUNTER
\LLS\15\RECOVER LSBS
\STA\T1WC
*
T103\LDA\CHKL\WAIT UNTIL ISR ADDS BUFFER TO CHECK LIST
\SZE
\JMP\T104\WE'RE DONE
\IRS\T1WC\STEP COUNTER
\JMP\T103\LOOP
\HLT\\TIMEOUT WAITING FOR TRANSFER TO COMPLETE
*
T104\STA\0\INTO X-REG
\JST\CBUF\CHECK BUFFER AS EXPECTED
\JST\LDEL\DELETE FROM
\DAC\CHKL\THE CHECK LIST
\JST\LADD\AND RETURN TO THE
\DAC\FREE\FREE BUFFER LIST
*
\LDA\PTDR\PICK UP DIRECTION CONTROL
\TCA\\SWITCH READ/WRITE
\STA\PTDR
\SPL\\DONE THE READ?
\JMP\T102\NO - GO DO THE READ
\LDA\CHNN\DONE WRITE AND READ - STEP CHANNEL
\AOA\\INCREMENT
\CAS\DMCN\COMPARE TO NUMBER OF CHANNELS
\HLT\\SHOULD NEVER EXCEED
\SKP\\WE'VE TRIED THEM ALL
\JMP\T101\LOOP OVER CHANNELS
*
\JMP*\TST1
*
T1WC\BSS\1
\EJCT
************************************************************
*
*    TEST 2 - MULTIPLE READS AND WRITES
*
TST2\DAC\**
\LDA\=2\TEST NUMBER
\STA\TNUM
*
*    PRE-PREPARE AS MANY BUFFERS AS POSSIBLE
*
T201\LDA\FREE\ANY FREE BUFFERS?
*
\SNZ\\YES - PREPARE IT
\JMP\T202\NO, ALL DONE
\STA\0\INTO X-REG
\JST\BRNG\DECIDE TRANSFER DIRECTION
\JST\IBUF\INITIALISE THE BUFFER
\JST\LDEL\DELETE FROM
\DAC\FREE\FREE BUFFER LIST
\JST\LADD\ADD TO
\DAC\RDYL\BUFFERS THAT ARE READY LIST
\JMP\T201\LOOP FOR AS MANY BUFFERS AS POSSIBLE
*
T202\CRA\\CLEAR THE 
\STA\FCHL\FREE CHANNEL LIST
*
\STA\PTDR\RANDOM DIRECTION CONTROL WORD
*
\LDA\T2TC\TIMING CONTROL FOR TEST2
\JST\ICTC\SET ON ALL CHANNELS
*
\LDA\T2ID\ADDRESS OF CALL-BACK
\STA\ICDR\SET-UP
\EJCT
*
*    SET-UP TO START A TRANSFER ON EACH CHANNEL
*
\LDA\MDMC\-VE NUMBER OF CHANNELS
\STA\TMP1\SET-UP CHANNEL COUNTER
\LDA\DCDA\POINT AT FIRST CHANNEL STRUCTURE
\ADD\DCBO\OFFSET TO BUFFER STRUCTURE ENTRY
\STA\TMP2\POINTER TO CHANNEL STRUCT BUFFER PTR
T203\LDA\RDYL
\SNZ\\IS THERE A BUFFER READY
\HLT\\NO - THAT'S NOT EXPECTED
\STA\0
\STA*\TMP2\POINT CHANNEL STRUCT AT BUFFER
\JST\LDEL\DELETE BUFFER
\DAC\RDYL\FROM READY LIST
\LDA\TMP2
\ADD\DSSZ\STEP BY SIZE OF CHANNEL STRUCTURE
\STA\TMP2
\IRS\TMP1\CHANNEL COUNTER
\JMP\T203\LOOP
*
\LDA\T2NT\INITIALIZE THE
\STA\T2CT\TRANSFER COUNTER
\EJCT
*
*    AND NOW START THEM
*
\LDA\MDMC\-VE NUMBER OF CHANNELS
\STA\TMP1\SET-UP CHANNEL COUNTER
\LDA\DCDA\POINT AT FIRST CHANNEL STRUCTURE
\ADD\DCBO\OFFSET TO BUFFER STRUCTURE ENTRY
\STA\TMP2\POINTER TO CHANNEL STRUCT BUFFER PTR
\CRA\\START AT CHANNEL NUMBER ZERO
\STA\TMP3
\LDA\DMCA\START ADDRESS FOR DMC1 IN LOW MEM
\STA\TMP4
\INH\\WANT TO START THEM ALL BEFORE INTERRUPT
*
T204\LDX*\TMP2\BUFFER POINTER
\LDA\TMP3
\OTA\ODCH\SET CHANNEL NUMBER
\JMP\*-1
\LDA\TMP4\POINT AT CHANNELS START ADDRESS LOCATION
\JST\STTF\START THE TRANSFER
*
\IRS\TMP4\STEP POINTER TO START ADDRESS
\IRS\TMP4\TWICE - STEP OVER END ADDRESS
\IRS\TMP3\STEP H/W CHANNEL NUMBER
\LDA\TMP2
\ADD\DSSZ\STEP BY SIZE OF CHANNEL STRUCTURE
\STA\TMP2
\IRS\TMP1\CHANNEL COUNTER
\JMP\T204\LOOP
*
\ENB\\HAVING STARTED THAT LOT, ALLOW INTERRUPTS
\EJCT
*
*    WAIT FOR WORK TO DO
*
T205\LDA\CHKL\IS THERE A BUFFER
\SZE\\WAITING TO BE CHECKED?
\JMP\T206\YES
*
\JMP\T205\LOOP (WILL CHECK FOR DMC DONE)
*
T206\STA\0\CHECK THIS BUFFER, AND RECYCLE
*
*
*\INH
*\JST\PLST
*\DAC\CHKL
*\JST\CRLF
*\ENB
*
*
\JST\LDEL\DELETE FROM THE
\DAC\CHKL\CHECK LIST
\JST\CBUF\CHECK IT WAS OK
*
\JST\BRNG\DECIDE TRANSFER DIRECTION
\JST\IBUF\INITIALISE THE BUFFER
\LDA\FCHL\IS THERE A FREE CHANNEL?
\SZE\\NO
\JMP\T208\YES - START THE TRANSFER
T207\JST\LADD\ADD TO THE
\DAC\RDYL\READY BUFFER LIST
\SKS\SKNB\ANY CHANNEL BUSY
\JMP\T205\YES, LOOP...
\JMP*\TST2\NO - DONE
*
T208\INH\\INTERRUPT CAN STEP T2CT AS WELL
*
*
\LDA\T2CT\GET TRANSFER COUNTER
\JST\POCT
\JST\CRLF
*
*
\LDA\T2CT\GET TRANSFER COUNTER
\SPL\\ALREADY EXHAUSTED?
\AOA\\NO, THEN STEP
\STA\T2CT
\SMI\\DONE?
\JMP\T207\YES - START NO MORE (ENB IN LADD)
\ENB\\RE-ENABLE
*
\LDA\FCHL\GET FREE CHANNEL
\SNZ\\SINCE TESTED ABOVE...
\HLT\\ASSERT - EXPECTED
\STA\TMP2\SAVE POINTER TO CHANNEL STRUCTURE
\SUB\DCDA\BASE OF DMC CHANNEL ARRAY
\IAB\\INTO B
\CRA\\A,B IS OFFSET IN BUFFER TABLE
\DIV\DSSZ\CALCULATE CHANNEL NUMBER
\INH\\PREVENT INTERRUPTION
\OTA\ODCH\SET CHANNEL NUMBER
\JMP\*-1
\STA\TMP1
\ADD\TMP1\DOUBLE
\ADD\DMCA\ADDRESS OF DMC START LOCATION
\JST\STTF\START THE TRANSFER
\LDX\TMP2\POINT AT CHANNEL STRUCTURE
\JST\LDEL\DELETE FROM
\DAC\FCHL\THE FREE CHANNEL LIST
* NOTE ENB IN LDEL
\JMP\T205\LOOP
*
DCBO\DAC\DCBS
\EJCT
*
*    TEST 2 - INTERRUPT COMPLETION CALLBACK
*
T2ID\DAC\T2IC
T2IC\DAC\**
\STX\T2IX\SAVE POINTER TO CHANNEL STRUCT
\LDA\DCBS,1\GET THE BUFFER STRUCTURE
\STA\0\INTO X
*
\JST\ILAD\ADD TO
\DAC\CHKL\THE CHECK LIST
\LDX\T2IX\RECOVER POINTER TO CHANNEL STRUCT
*
\LDA\RDYL\IS THERE A BUFFER READY TO GO?
\SZE
\JMP\T2I2\YES
T2I1\JST\ILAD\NO - PUSH CHANNEL ONTO
\DAC\FCHL\THE FREE CHANNEL LIST
\JMP*\T2IC
*
T2I2\LDA\T2CT\GET TRANSFER COUNTER
\SPL\\ALREADY EXHAUSTED?
\AOA\\STEP
\STA\T2CT
\SMI\\DONE?
\JMP\T2I1\YES - START NO MORE
*
\LDA\RDYL\IS THERE A BUFFER READY TO GO?
\SNZ
\HLT
\STA\DCBS,1\POINTER TO BUFFER STRUCT IN CHN STRUCT
\STA\0
\JST\ILDL\REMOVE BUFFER FROM
\DAC\RDYL\THE READY BUFFER LIST
\LDA\ICHA\GET THE CHANNEL NUMBER
\ADD\ICHA\DOUBLE IT
\ADD\DMCA\CALCULATE START ADDRESS LOCATION
\JST\STTF\START THE TRANSFER
\JMP*\T2IC
*
T2IX\BSS\1
*
*T2TC\OCT\100200\RANDOM 128 TO 383 CYCLES BETWEEN REQUESTS
T2TC\OCT\122000\RANDOM 1024 TO 2047 CYCLES BETWEEN REQUESTS
T2NT\DAC\NDMC-1000\NUMBER OF TRANSFERS LESS NDMC FIRST STARTED
T2CT\BSS\1\TRANSFER COUNTER
\EJCT
************************************************************
*    INITIALIZE ALL CHANNEL'S TIMING CONTROL WORDS
*    AND TIMING RANDOM NUMBER GENERATORS
*
ICTC\DAC\**
\STA\ICTD
\LDX\MDMC\-VE NUMBER OF DMC CHANNELS
\CRA\\CHANNEL NUMBER
*
ICT1\OTA\ODCH\SET CHANNEL NUMBER
\JMP\*-1
\IAB\\SAVE CHANNEL NUMBER
\LDA\ICTD
\OTA\OTCW\TIMING CONTROL TO H/W
\JMP\*-1
\JST\PRNG\GET A SEED TO USE
\OTA\OTRN\RANDOM TIMING SEED TO H/W
\JMP\*-1
*
\IAB\\RECOVER CHANNEL NUMBER
\AOA\\STEP
\IRS\0\STEP COUNTER
\JMP\ICT1
*
\JMP*\ICTC
*
ICTD\BSS\1
\EJCT
************************************************************
*    INTERRUPT SERVICE ROUTINE
*
INTR\DAC\**
\STA\SAVA\SAVE CPU REGISTERS...
\INK
\STA\SAVK
\STX\SAVX
\IAB
\STA\SAVB
*
\SKS\SKNI\SKIP IF VDMC NOT INTERRUPTING
\SKP
\HLT\\OOPS - WHAT DID INTERRUPT THEN?
*
*    HERE WHEN WE KNOW VDMC INTERRUPTED
*
\INA\IDCH\SAVE VDMC CHANNEL REGISTER
\JMP\*-1
\STA\SAVC
*
\INA\IICH\READ HIGHEST PRI. INTR. CHANNEL
\JMP\*-1
\STA\ICHA
\OTA\ODCH\AND SELECT IT
\JMP\*-1
\OCP\CCIN\CLEAR CHANNEL'S INTERRUPT
*
\INA\IERR\GET ERRORS
\JMP\*-1
\SZE\\SKIP IF NONE
\HLT\\OOPS...
*
\LDA\ICHA
*
\MPY\DSSZ\CALCULATE OFFSET INTO CHANNEL DATA STRUCTURE ARRAY
\IAB\\GET LSBS
\ADD\DCDA\ADD START OF ARRAY
\STA\0\INTO X
*
\LDA\ICDR\ADDRESS OF INTERRUPT CHANNEL DONE ROUTINE
\SZE\\IS IT SET?
\JMP\INT2\YES - CALL IT
\EJCT
*
*    DEFAULT DMC CHANNEL DONE BEHAVIOUR
*
\LDA\DCBS,1\GET BUFFER ADDRESS USED BY THIS CHANNEL
\STA\0\INTO X
*
\JST\ILAD\ADD TO
\DAC\CHKL\THE CHECK LIST
*
INT1\LDA\SAVC\RESTORE VDMC CHANNEL REGISTER
\OTA\ODCH
\JMP\*-1
*
\LDA\SAVB\RESTORE CPU REGISTERS...
\IAB
\LDX\SAVX
\LDA\SAVK
\OTK
\LDA\SAVA
\ENB\\RE-ENABLE INTERRUPTS
\JMP*\INTR\AND RETURN
*
*    CALL ICDR (X POINTING AT CHANNEL STRUCT)
*
INT2\JST*\ICDR\CALL REGISTERED ROUTINE
\JMP\INT1
*
SAVA\BSS\1\SAVE A-REG
SAVB\BSS\1\SAVE B-REG
SAVK\BSS\1\SAVE KEYS
SAVX\BSS\1\SAVE X-REG
SAVC\BSS\1\SAVE VDMC CHANNEL NUMBER
ICHA\BSS\1\CHANNEL NUMBER BEING SERVICED
ICDR\DAC\0\ADDRESS OF INTERRUPT CHANNEL DONE ROUTINE
\EJCT
************************************************************
*
*    GENERAL TYPE ROUTINE
*
TYPE\DAC\**\TYPE ROUT
\STA\TMP1\SAVE MSG ADRS
\LDA*\TMP1\GET WORD COUNT
\STA\TMP2\
\SKS\'104\TEST FOR ASR BUSY
\JMP\*-1
\OCP\'104\ENABLE, OUTPUT MODE
PRCH\IRS\TMP1\BUMP ADDR FOR NEXT WORD OF MSG.
\LDA*\TMP1
\ALR\8
\OTA\'4\OUTPUT 1-8 IN ASCII
\JMP\*-1
\LDA*\TMP1
\OTA\'4\OUTPUT 9-16
\JMP\*-1
\IRS\TMP2\SKIP AFTER LAST WORD
\JMP\PRCH\PRINT NEXT WORD
\JMP*\TYPE
\EJCT
************************************************************
*
*    LIST MANIPULATION ROUTINES
*
* THESE ROUTINES MANIPULATE A DOUBLY LINKED LIST
*
* EACH ENTRY IN THE LIST STARTS WITH TWO POINTERS
* - FWD (FORWARD) POINTER TO THE NEXT ENTRY
* - BWD (BACKWARD) POINTER TO THE PREVIOUS ENTRY
*
* THE FWD POINTER OF THE LAST ENTRY IN THE LIST IS
* SET TO ZERO.
* THE BWD POINTER OF THE FIRST ENTRY IN THE LIST
* POINTS TO THE LAST ENTRY IN THE LIST.
*
* +------+    +-----+    +-----+     +-----+    +-----+
* | HEAD |--->| FWD |--->| FWD | ... | FWD |--->|  0  |<--+
* +------+    +-----+<-+ +-----+     +-----+<-+ +-----+   |
*          +--| BWD |  +-| BWD | ... | BWD |  +-| BWD |   |
*          |  +-----+    +-----+     +-----+    +-----+   |
*          |                                              |
*          +----------------------------------------------+           
*
* NOTE THAT THE BWD POINTERS POINT AT THE FIRST LOCATION
* (THE FWD POINTER) OF THE ENTRY THEY POINT TO.
*
* EACH ROUTINE IS PROVIDED IN TWO VERSIONS, ONE WHICH IS
* INTENDED TO BE USED FROM AN INTERRUPT SERVICE ROUTINE, AND
* A SECOND THAT MAY SAFELY BE USED IN NORMAL (NON-INTERRUPT)
* CODE.
*
\EJCT
************************************************************
*
* LADD - LIST ADD - ADD ENTRY TO END OF LIST
*
* CALLING CONVENTION
*
*    (ENTER WITH NEW ENTRY POINTED TO BY X)
*    JST LADD
*    DAC HEAD  <- PTR TO HEAD PTR OF LIST TO ADD TO
*    ...       <- RETURN HERE
*
* A, B, AND X REGISTERS ARE ALL PRESERVED
* INTERRUPTS ARE DISABLED FOR THE DURATION OF THE
* CRITICAL SECTION THAT MANIPULATES THE LIST DATA
* STRUCTURE.
*
LADD\DAC\**\ADD ITEM TO END OF LIST
\STA\LSVA\SAVE A
\LDA*\LADD\PICK UP POINTER TO HEAD POINTER
\IRS\LADD\SKIP OVER POINTER TO HEAD POINTER
\INH\\DISABLE INTERRUPTS
\JST\XLAD\CALL ACTUAL ROUTINE
\LDA\LSVA\RESTORE A
\ENB\\RE-ENABLE
\JMP*\LADD
*
LSVA\BSS\1\SAVE A, SHARED WITH LDEL
*
* ILAD - INTERRUPT ROUTINE LIST ADD
*
* SAME CALLING CONVENTION AS LADD.
*
ILAD\DAC\**
\STA\LISV\SAVE A
\LDA*\ILAD\PICK UP POINTER TO HEAD POINTER
\IRS\ILAD\SKIP OVER POINTER TO HEAD POINTER
\JST\XLAD\CALL ACTUAL ROUTINE
\LDA\LISV\RESTORE A
\JMP*\ILAD
*
LISV\BSS\1\SAVE A, SHARED WITH ILDL
\EJCT
*
* XLAD - ACTUAL LIST ADDITION ROUTINE
*        NOT TO BE CALLED DIRECTLY
*        (EXCEPT BY LADD OR ILAD)
*
* ON ENTRY:
*    A: POINTER TO HEAD POINTER
*    X: POINTER TO ENTRY TO ADD TO LIST
*
XLAD\DAC\**
\STA\XLHP\SAVE POINTER TO HEAD POINTER
\LDA*\XLHP\GET HEAD POINTER
\SNZ\\IS LIST EMPTY?
\JMP\XLA2\YES, SPECIAL CASE
\AOA\\NO, FIND END OF LIST
\STA\XLHP\NOW POINTING TO 1ST ENTRY'S BACK POINTER
\LDA*\XLHP\PICK THAT UP
\STX*\XLHP\MAKE THAT POINTER POINT AT NEW ENTRY
\STA\XLHP\POINT AT OLD LAST ENTRY
\STX*\XLHP\MAKE THAT POINTER POINT AT NEW ENTRY
XLA1\STA\LBWD,1\NEW ENTRY'S BACK POINTER POINTS AT OLD LAST ENTRY
\CRA
\STA\LFWD,1\NEW LAST ENTRY'S FORWARD POINTER IS NULL
\JMP*\XLAD
*
XLA2\STX*\XLHP\POINT HEAD AT NEW ENTRY
\LDA\0\POINTER TO NEW ENTRY
\JMP\XLA1
*
XLHP\BSS\1\TEMPORARY, NOTIONALLY POINTER TO HEAD POINTER
*\\\SHARED WITH XLDL
\EJCT
************************************************************
*
* LDEL - LIST DELETE - DELETE ENTRY FROM LIST
*
* CALLING CONVENTION
*
*    (ENTER WITH ENTRY TO BE DELETED POINTED TO BY X)
*    JST LDEL
*    DAC HEAD  <- PTR TO HEAD PTR OF LIST
*    ...       <- RETURN HERE
*
* A, B, AND X REGISTERS ARE ALL PRESERVED
* INTERRUPTS ARE DISABLED FOR THE DURATION OF THE
* CRITICAL SECTION THAT MANIPULATES THE LIST DATA
* STRUCTURE.
*
LDEL\DAC\**\DELETE ITEM FROM LIST
\STA\LSVA\SAVE A
\LDA*\LDEL\PICK UP POINTER TO HEAD POINTER
\IRS\LDEL\SKIP OVER POINTER TO HEAD POINTER
\INH\\DISABLE INTERRUPTS
\JST\XLDL\CALL ACTUAL ROUTINE
\LDA\LSVA\RESTORE A
\ENB\\RE-ENABLE INTERRUPTS
\JMP*\LDEL
*
* ILDL - INTERRUPT ROUTINE LIST DEL
*
* SAME CALLING CONVENTION AS LDEL.
*
ILDL\DAC\**
\STA\LISV\SAVE A
\LDA*\ILDL\PICK UP POINTER TO HEAD POINTER
\IRS\ILDL\SKIP OVER POINTER TO HEAD POINTER
\JST\XLDL\CALL ACTUAL ROUTINE
\LDA\LISV\RESTORE A
\JMP*\ILDL
\EJCT
*
* XLDL - ACTUAL LIST DELETION ROUTINE
*        NOT TO BE CALLED DIRECTLY
*        (EXCEPT BY LDEL OR ILDL)
*
* ON ENTRY:
*    A: POINTER TO HEAD POINTER
*    X: POINTER TO ENTRY TO DELETE FROM LIST
*
XLDL\DAC\**\POINTER TO HEAD POINTER IN A
\STA\XLHP\SAVE IT
\LDA*\XLHP\GET THE HEAD POINTER
\CAS\0\IS IT THE FIRST ENTRY THAT IS TO BE DELETED?
\SKP\\NO
\JMP\XLD2\YES, NEED TO CHANGE THE HEAD POINTER
\LDA\LBWD,1\NO, GET LBWD POINTER OF ENTRY TO BE DELETED
XLD1\STA\XLTP\SAVE IN XLTP
\LDA\LFWD,1\GET LFWD OF ENTRY TO BE DELETED
\STA*\XLTP\PUT INTO LFWD OF PREVIOUS ENTRY (OR HEAD)
\SNZ\\IS THE LAST ENTRY IN LIST BEING DELETED?
\LDA*\XLHP\YES, POINT AT FIRST ENTRY
\SNZ\\MIGHT BE ZERO IF WE JUST DELETED THE
\JMP*\XLDL\ONLY ENTRY, IN WHICH CASE WE'RE DONE
\AOA\\POINT AT LBWD OF NEXT ENTRY (OR FIRST)
\STA\XLTP\SAVE IN TEMPORARY
\LDA\LBWD,1\LBWD POINTER OF ENTRY TO BE DELETED
\STA*\XLTP\PUT IN LBWD POINTER OF NEXT ENTRY
\JMP*\XLDL
*
XLD2\LDA\XLHP\POINT AT HEAD POINTER
\JMP\XLD1\INSTEAD OF LBWD,1
*
XLTP\BSS\1
\EJCT
************************************************************
*
*    DEBUG ROUTINES
*
POCT\DAC\**
\STA\POT1\SAVE IN TEMPORARY
\LDA\=-6
\IMA\POT1
\IAB
\CRA
\LLL\1
\JMP\POC2
POC1\CRA
\LLL\3
POC2\ADD\='260
\JST\PCHR
\IRS\POT1
\JMP\POC1
\JMP*\POCT
*
POT1\BSS\1
*
PCHR\DAC\**
\SKS\'104\TEST FOR ASR BUSY
\JMP\*-1
\OCP\'104\ENABLE, OUTPUT MODE
\OTA\4
\JMP\*-1
\JMP*\PCHR
*
PSPC\DAC\**
\LDA\='240
\JST\PCHR
\JMP*\PSPC
*
CRLF\DAC\**
\LDA\='105215
\JST\PCHR
\ICA
\JST\PCHR
\JMP*\CRLF
\EJCT
************************************************************
*
*    PRINT A LIST
*
PLST\DAC\**\POINTER TO HEAD AT RETURN ADDRESS
\STX\PLSX
\LDX*\PLST
\IRS\PLST
\JST\CRLF
\LDA\0\POINTER TO HEAD
\JST\POCT
\JST\PSPC
\LDX*\0
\LDA\0\HEAD POINTER ITSELF
\JST\POCT
\JST\CRLF
PLS1\LDA\0
\SNZ\\ZERO?
\JMP\PLS2\DONE
\JST\POCT
\JST\PSPC
\LDA\LFWD,1
\JST\POCT
\JST\PSPC
\LDA\LBWD,1
\JST\POCT
\JST\CRLF
\LDX*\0
\JMP\PLS1
*
PLS2\LDX\PLSX
\JMP*\PLST
*
PLSX\BSS\1
\EJCT
************************************************************
*    RANDOM NUMBER GENERATION
*
LFSR\DAC\**
\LGR\1\SHIFT RIGHT ONE, LSB TO C
\SRC\\LSB WAS ONE?
\ERA\TAPS\YES, XOR WITH POLYNOMIAL
\JMP*\LFSR
*
TAPS\OCT\122532\(0xA55A)
*
*    SYSTEM-WIDE "MASTER" RANDOM NUMBER GENERATOR
*
PRNG\DAC\**
\LDA\SSEE
\JST\LFSR
\IMA\SSEE
\JMP*\PRNG
*
SSEE\OCT\177777\SYSTEM SEED
*
*    BUFFER SPECIFIC RANDOM NUMBER
*
*    ENTER WITH X POINTING AT BUFFER STRUCTURE
*
BRNG\DAC\**
\LDA\SEED,1
\JST\LFSR
\IMA\SEED,1
\JMP*\BRNG
\EJCT
************************************************************
*
*    INITIALIZE A BUFFER READY FOR TRANSFER
*
*    ENTER WITH X POINTING AT A BUFFER STRUCTURE
*    ENTER WITH A MSB INDICATING TRANSFER DIRECTION
*
IBUF\DAC\**
\ANA\='100000\ISOLATE TOP BIT OF A
\IMA\DSTR,1\SAVE, GET DMC START ADDRESS
\SSP\\LOSE TOP BIT
\ERA\DSTR,1\OR-IN RANDOM DIRECTION BIT
\IMA\DSTR,1\SAVE DMC START, GET DIRN. BIT
\IMA\DEND,1\ALSO INTO DMC END ADDRESS
\SSP\\IN CASE USING HIGH SPEED DMC
\ERA\DEND,1
\STA\DEND,1
*
\LDA\SEED,1\GET SEED
\STA\DPRN,1\SAVE SEED TO TELL VDMC DEVICE
*
\LDA\DSTR,1\GET START ADDRESS (DIRECTION BIT)
\SPL\\ARE WE READING OR WRITING?
\JMP\IBWR\WE ARE WRITING (INPUT FROM PERIPHERAL)
*
* INITIALIZE FOR READING (OUTPUT TO PERIPHERAL)
*
* CLEAR INITIAL GUARD REGION TO ZERO,
* INITIALIZE TRANSFER WITH RNG AND CHECKSUM,
* CLEAR FINAL GUARD REGION TO ZERO
*
\LDA\BSTR,1\GET BUFFER START
\STA\TMP1
\LDA\MBGD\-VE SIZE OF BUFFER GUARD
\STA\TMP2
\CRA
\STA\CSUM
IBU1\STA*\TMP1\CLEAR INITIAL GUARD AREA
\IRS\TMP1\STEP POINTER
\IRS\TMP2\STEP COUNTER
\JMP\IBU1
*
* AT THIS POINT SHOULD BE AT DMC START - CHECK
\LDA\DSTR,1
\SSP\\LOSE DIRECTION BIT
\ERA\TMP1
\SZE
\HLT\\OOPS
*
\LDA\DSTR,1\CALCULATE -VE TRANSFER SIZE - 1
\SUB\DEND,1\AS START - END
\STA\TMP2
IBU2\LDA\SEED,1\INITIALIZE TRANSFER AREA
\JST\LFSR
\IMA\SEED,1
\STA*\TMP1
*
\IMA\CSUM\GET CHECKSUM (SAVE CURRENT WORD)
\ARR\1\ROTATE RIGHT
\ADD\CSUM\ADD IN CURRENT WORD
\STA\CSUM\SAVE CHECKSUM
*
\IRS\TMP1\STEP POINTER
\IRS\TMP2\STEP COUNTER
\JMP\IBU2
*
* LAST WORD IS CHECKSUM
*
\ARR\1\ROTATE RIGHT
\TCA
\STA*\TMP1
\IRS\TMP1\STEP POINTER
*
* AT THIS POINT SHOULD BE ONE AFTER DMC END
\LDA\DEND,1
\AOA\\ADD ONE TO MATCH POINTER
\SSP\\LOSE DIRECTION BIT
\ERA\TMP1
\SZE
\HLT\\OOPS
*
\LDA\MBGD\-VE SIZE OF BUFFER GUARD
IBU3\STA\TMP2
\CRA
IBU4\STA*\TMP1\CLEAR FINAL GUARD AREA
\IRS\TMP1\STEP POINTER
\IRS\TMP2\STEP COUNTER
\JMP\IBU4
*
* CHECK WHOLE BUFFER INITIALIZED
\LDA\TMP1
\SUB\BSTR,1\SUBTRACT START ADDRESS
\ERA\BSIZ,1
\SZE
\HLT\\OOPS
*
\JMP*\IBUF\WE'RE DONE
*
* INITIALIZE FOR WRITING (INPUT FROM PERIPHERAL)
*
* JUST CLEAR WHOLE BUFFER, INCLUDING INITIAL AND
* FINAL BUFFER GUARD REGIONS.
*
IBWR\LDA\BSTR,1\GET BUFFER START
\STA\TMP1
\LDA\BSIZ,1\GET BUFFER SIZE
\TCA
\JMP\IBU3
\EJCT
************************************************************
*
*    CHECK A BUFFER AFTER A TRANSFER
*
CBUF\DAC\**
\LDA\BSTR,1\GET BUFFER START
\STA\TMP1
\LDA\MBGD\-VE SIZE OF BUFFER GUARD
\STA\TMP2
CBU1\LDA*\TMP1\CHECK INITIAL GUARD AREA
\SZE\\SHOULD BE ZERO
\HLT\\OOPS...
\IRS\TMP1\STEP POINTER
\IRS\TMP2\STEP COUNTER
\JMP\CBU1
*
\LDA\DPRN,1\GET SEED THAT THE VDMC DEVICE USED
\STA\SEED,1
\LDA\DSTR,1\CALCULATE -VE TRANSFER SIZE -1
\SUB\DEND,1\AS (START-END)
\STA\TMP2
\CRA
\STA\CSUM
*
CBU2\LDA\SEED,1\CHECK TRANSFER AREA
\JST\LFSR
\IMA\SEED,1
*
\IMA\CSUM\GET CHECKSUM (SAVE CURRENT WORD)
\ARR\1\ROTATE RIGHT
\ADD\CSUM\ADD IN CURRENT WORD
\IMA\CSUM\SAVE CHECKSUM, RECOVER WORD
*
\ERA*\TMP1\COMPARE TO DATA IN BUFFER
\SZE\\SHOULD BE ZERO
\HLT\\OOPS...
\IRS\TMP1\STEP POINTER
\IRS\TMP2\STEP COUNTER
\JMP\CBU2
*
* LAST WORD IS CHECKSUM...
*
\LDA\CSUM\GET CHECKSUM
\ARR\1\ROTATE RIGHT
\TCA
\ERA*\TMP1\COMPARE TO DATA IN BUFFER
\SZE\\SHOULD BE ZERO
\HLT\\OOPS...
\IRS\TMP1\STEP POINTER
*
\LDA\MBGD\-VE SIZE OF BUFFER GUARD
\STA\TMP2
CBU3\LDA*\TMP1\CHECK FINAL GUARD AREA
\SZE\\SHOULD BE ZERO
\HLT\\OOPS...
\IRS\TMP1\STEP POINTER
\IRS\TMP2\STEP COUNTER
\JMP\CBU3
*
\JMP*\CBUF
\EJCT
************************************************************
*    PREPARE (AND OPTIONALLY START) A TRANSFER
*
*    ENTER WITH A-REG HOLDING THE DMC CHANNEL NUMBER &
*    LOCATION PTST NON-ZERO TO START THE TRANSFER
*    LOCATION PTDR INDICATING DIRECTION OF TRANSFER:
*    -VE  - READ  (PERIPHERAL TO CPU)
*    +VE  - WRITE (CPU TO PERIPHERAL)
*    ZERO - RANDOMLY DECIDE DIRECTION
*
*    EXIT WITH DMC START ADDRESS IN A-REG (WHICH CAN BE USED
*    TO SUBSEQUENTLY DECIDE ON WHETHER AN INPUT OR OUTPUT
*    TRANSFER IS REQUIRED).
*    AND BUFFER STRUCTURE POINTED TO BY X
************************************************************
PTFR\DAC\**
\STA\PTCN\STORE THE CHANNEL NUMBER - 1 (0-15)
\ADD\PTCN\DOUBLE IT
\ADD\DMCA\ADD ADDRESS OF CHANNEL 1 START ADDR.
\STA\PTDA\STORE COMPUTE DMC ADDRESS
\LDA\PTCN\RETRIEVE CHANNEL NUMBER
\MPY\DSSZ\CALCULATE OFFSET INTO CHANNEL DATA STRUCTURE ARRAY
\IAB\\GET LSBS
\ADD\DCDA\COMPUTE ADDRESS OF...
\STA\PTDS\DMC CHANNEL DATA STRUCTURE
*
\LDA\FREE\ARE THERE ANY FREE BUFFERS?
\SNZ\\SKIP IF THERE ARE
\HLT\\OOPS...
\LDX*\FREE\GET FIRST AVAILABLE BUFFER
\JST\LDEL\DELETE...
\DAC\FREE\FROM THE FREE LIST
*
\LDA\PTDR\DIRECTION INDICATOR
\SNZ\\NON-ZERO - USE AS DIRECTION BIT
\JST\BRNG\BUT ZERO - RANDOMLY DECIDE
\JST\IBUF\INITIALISE THE BUFFER
\LDA\PTDS\POINT AT DMC CHN. STRUCTURE
\IMA\0\GET BUFFER POINTER
\STA\DCBS,1\SAVE IN DMC CHN. STRUCTURE
\STA\0\REINSTATE X-REG PTR. TO BUFFER STRUCTURE
*
\LDA\PTCN\RECOVER CHANNEL NUMBER
\INH\\ABOUT TO USE H/W CHANNEL SELECT REG
\OTA\ODCH\SELECT THE CHANNEL TO ADDRESS
\JMP\*-1
*
\LDA\PTST\GET START TRANSFER FLAG
\SNZ\\NON-ZERO - START THE TRANSFER
\JMP\PT01
\LDA\PTDA\ADDRESS OF CHANNEL START ADDRESS PTR
\JST\STTF\START THE TRANSFER
*
PT01\ENB\\DONE WITH THE H/W CHANNEL SELECT REG
\LDA\DSTR,1\DMC START ADDRESS
\JMP*\PTFR
*
PTST\BSS\1\START TRANSFER FLAG
PTDR\BSS\1\TRANSFER DIRECTION CONTROL
PTCN\BSS\1\CHANNEL NUMBER
PTDS\BSS\1\DMC CHANNEL DATA STRUCTURE
PTDA\BSS\1\POINTER TO DMC START ADDRESS
*
*    START A TRANSFER
*
*    ENTER WITH X POINTING AT THE BUFFER STRUCT
*    A HOLDING AN ADDRESS POINTING AT THE DMC
*      CHANNEL'S START ADDRESS LOCATION IN LOW MEM.
*
*    THE H/W VDMC CHANNEL SELECT REGISTER MUST
*    BE INITIALIZED, AND THIS ROUTINE MUST BE
*    CALLED IN A CONTEXT WHERE IT WILL NOT CHANGE
*    (CONSIDER DISABLING INTERRUPTS)
*
STTF\DAC\**
\STA\STDA\SAVE DMC CHANNEL START ADDRESS PTR
\LDA\DSTR,1\DMC START ADDRESS
\STA*\STDA\SET UP CHANNEL'S DMC START ADDRESS
\IRS\STDA\INCREMENT TO POINT AT END ADDRESS
\LDA\DEND,1\DMC END ADDRESS
\STA*\STDA\SET UP CHANNEL'S DMC START ADDRESS
\SUB\DSTR,1\CALCULATE SIZE
\AOA\\OF TRANSFER
\ANA\='077777\LOSE THE DIRECTION BIT
*
\OTA\OTFS\TELL VDMC THE TRANSFER SIZE
\JMP\*-1
\LDA\DPRN,1\INITIALIZE THE H/W DATA RNG
\OTA\ODRN
\JMP\*-1
\LDA\DSTR,1\MSB=1 INDICATES AN INPUT TRANSFER
\CAS\=0\COMPARE TO ZERO TO DECIDE SIGN
\OCP\STOT\A>0 START AN OUTPUT TRANSFER
\SKP\\(A IS NEVER ZERO)
\OCP\STIT\START AN INPUT TRANSFER
\JMP*\STTF
*
STDA\BSS\1
\EJCT
************************************************************
*
*    INTERESTING INSTRUCTIONS FOR DMC TO INTERRUPT
*
************************************************************
ITRS\DAC\**
\LDX\NRMT\LOAD -VE SIZE OF NRM OPERAND TABLE
\LDA\=29\FIRST EXPECTED RESULT
ITR1\STA\ITT1
*
\LDA\NRMT+1,1\GET +VE OPERAND LSB
\IAB
\LDA\NRMT,1\GET +VE OPERAND MSB
\NRM\\TEST NORMALIZATION
\SCA\\GET SHIFT COUNT
\CAS\ITT1
\HLT\\TOO BIG
\SKP\\JUST RIGHT
\HLT\\TOO SMALL
\LDA\NRMT+3,1\GET -VE OPERAND LSB
\IAB
\LDA\NRMT+2,1\GET -VE OPERAND MSB
\NRM\\TEST NORMALIZATION
\SCA\\GET SHIFT COUNT
\CAS\ITT1
\HLT\\TOO BIG
\SKP\\JUST RIGHT
\HLT\\TOO SMALL
*
\LDA\0
\ADD\=4\STEP INDEX
\STA\0
\LDA\ITT1
\SUB\=1\DECREMENT EXPECTED RESULT
\SMI\\NEGATIVE
\JMP\ITR1\NO - LOOP FOR MORE
*
\JMP*\ITRS
*
ITT1\BSS\1
*
*    TABLE OF NRM OPERATION OPERAND
*
NRMS\OCT\000000,000001\29
\OCT\177777,077776\29
\OCT\000000,000003\28
\OCT\177777,077774\28
\OCT\000000,000007\27
\OCT\177777,077770\27
\OCT\000000,000017\26
\OCT\177777,077760\26
\OCT\000000,000037\25
\OCT\177777,077740\25
\OCT\000000,000077\24
\OCT\177777,077700\24
\OCT\000000,000177\23
\OCT\177777,077600\23
\OCT\000000,000377\22
\OCT\177777,077400\22
\OCT\000000,000777\21
\OCT\177777,077000\21
\OCT\000000,001777\20
\OCT\177777,076000\20
\OCT\000000,003777\19
\OCT\177777,074000\19
\OCT\000000,007777\18
\OCT\177777,070000\18
\OCT\000000,017777\17
\OCT\177777,060000\17
\OCT\000000,037777\16
\OCT\177777,040000\16
\OCT\000000,077777\15
\OCT\177777,000000\15
\OCT\000001,077777\14
\OCT\177776,000000\14
\OCT\000003,077777\13
\OCT\177774,000000\13
\OCT\000007,077777\12
\OCT\177770,000000\12
\OCT\000017,077777\11
\OCT\177760,000000\11
\OCT\000037,077777\10
\OCT\177740,000000\10
\OCT\000077,077777\9
\OCT\177700,000000\9
\OCT\000177,077777\8
\OCT\177600,000000\8
\OCT\000377,077777\7
\OCT\177400,000000\7
\OCT\000777,077777\6
\OCT\177000,000000\6
\OCT\001777,077777\5
\OCT\176000,000000\5
\OCT\003777,077777\4
\OCT\174000,000000\4
\OCT\007777,077777\3
\OCT\170000,000000\3
\OCT\017777,077777\2
\OCT\160000,000000\2
\OCT\037777,077777\1
\OCT\140000,000000\1
\OCT\077777,077777\0
\OCT\100000,000000\0
NRMT\DAC\NRMS-NRMT\-VE LENGTH OF TABLE
************************************************************
*
*    INITIALIZATION CODE THAT RUNS JUST ONCE
*
************************************************************
INIT\DAC\**\HERE SO RETURN ADDRESS NOT OVERWRITTEN
\JMP\INI2\JUMP TO MAIN INITIALIZATION CODE
*
*    END OF BUFFER INITIALISATION CODE
*    LOCATED HERE SO IT ISN'T OVERWRITTEN BY
*    BUFFERS OR DESCRIPTORS
*
IBS2\LDA\TMP2\POINT PAST END OF CURRENT BUFFER
\SUB*\TMP3\SUBTRACT BUFFER SIZE
\STA\TMP2\START OF BUFFER
*
*   BUILD BUFFER DESCRIPTOR
*
\STA\BSTR,1\ACTUAL START ADDRESS OF THIS BUFFER
\ADD\BGRD\ADD SIZE OF GUARD REGION TO CALCULATE
\STA\DSTR,1\START OF DMC BUFFER
\LDA*\TMP3\GET BUFFER SIZE
\STA\BSIZ,1\TOTAL SIZE OF THIS BUFFER
\ADD\BSTR,1\ADD START ADDR - POINTING JUST PAST BUFFER
\SUB\=1\POINT AT LAST WORD OF BUFFER
\SUB\BGRD\SUBTRACT SIZE OF GUARD REGION TO CALCULATE
\STA\DEND,1\END OF DMC BUFFER
*
\JST\PRNG\GET NEXT SYSTEM RANDOM NUMBER
\STA\SEED,1\AND USE AS SEED FOR THIS CHANNEL
\CRA\\CLEAR THE DPRN FOR NOW
\STA\DPRN,1
*
\JST\LADD\ADD TO FREE BUFFER LIST
\DAC\FREE
*
IBSX\LDA\TMP3\DECREMENT POINTER TO BUFFER SIZE
\SUB\=1
\STA\TMP3
\LDA\0\POINT AT NEXT (EARLIER) DESCRIPTOR
\SUB\BDSZ
\STA\0
\IRS\TMP1\STEP LOOP COUNTER
\JMP\IBS2
*
\JMP*\INIT
\EJCT
************************************************************
*    DMC CHANNEL DATA STRUCTURE
DCDC\SET\NDMC\COUNTER FOR ACTUAL NUMBER OF CHANNELS
*
DCDM\MAC\\ALLOCATE STORAGE FOR CHANNEL STRUCTURE
\IFN\DCDC\TEST COUNTER
\BSS\DCSZ\STORAGE
DCDC\SET\DCDC-1\DECREMENT COUNTER
\ENDC
\ENDM
*
DCDS\EQU\*\UP TO 16 DMC CHANNEL STRUCTURES
\DCDM\1
\DCDM\2
\DCDM\3
\DCDM\4
\DCDM\5
\DCDM\6
\DCDM\7
\DCDM\8
\DCDM\9
\DCDM\10
\DCDM\11
\DCDM\12
\DCDM\13
\DCDM\14
\DCDM\15
\DCDM\16
\EJCT
************************************************************
*    END OF MAIN PROGRAM
\FIN
*
*    EVERYTHING BEYOND THIS POINT IS OVERWRITTEN BY
*    BUFFERS AND THEIR DESCRIPTORS.
************************************************************
*    DMC BUFFER SIZE TABLE
*
*    53 (A PRIME NUMBER) ENTRIES SO ALWAYS CIRCULATES AROUND
*    THE DMC CHANNELS THAT ARE IN USE
*
*    ENTRIES SUM TO 32768 (2^15)
*
*    TABLE WILL BE OVERWRITTEN FIRST BY VALUES SCALED BY
*    THE ACTUAL AVAILABLE MEMORY, AND MODIFIED TO ALLOW
*    SPACE FOR A BUFFER DESCRIPTOR. THEN, AS THE
*    DESCRIPTORS ARE BUILT (LAST TO FIRST) THE FIRST FEW
*    DESCRIPTORS OVERWRITE THE TABLE.
*
DBFS\OCT\1002,1726,2003,1755,1015,1264,0740,0545
\OCT\1277,0324,2003,1277,1146,1210,0607,1120
\OCT\2045,1742,0574,1002,0532,1015,1444,1457
\OCT\1326,2134,1623,1521,0740,1562,0442,1562
\OCT\1341,0276,1367,0221,1402,0173,0607,0545
\OCT\0337,0442,1700,1341,1505,0337,0311,1416
\OCT\2134,0740,0247,0517,1576
DBFE\EQU\*
************************************************************
*   DATA USED ONLY IN INITIALISATION CODE
*
HIAD\BSS\1\HIGHEST ADDRESS IN MEMORY
NBUF\DAC\DBFE-DBFS\NUMBER OF BUFFERS TO ALLOCATE
MBUF\DAC\DBFS-DBFE\-VE NUMBER OF BUFFERS
PDBS\DAC\DBFS\POINT AT START OF DMC BUFFER TABLE
PDBE\DAC\DBFE\POINT AT END OF DMC BUFFER TABLE
PEND\DAC\AEND\POINT AT END OF PROGRAM
\EJCT
************************************************************
*
*    INITIALISATION CODE THAT GETS OVERWRITTEN BY BUFFERS
*
INI2\SZE\\IF A-REG ZERO THEN USE DEFAULT RANDOM SEED
\STA\SSEE\ELSE STORE A IN SEED
*
\LDA\DMCN\GET NUMBER OF DMC CHANNELS
\SNZ
\HLT\\NO DMC HARDWARE, STOP NOW
*
* CHECK THAT A VDMC HARDWARE BLOCK IS CONNECTED
*
\LDX\=-100\NUMBER OF ATTEMPTS TO TRY
\CRA\\CHANNEL NUMBER TO TRY AND WRITE
INI3\OTA\ODCH\TO THE CHANNEL SELECT REGISTER
\SKP\\DIDN'T RESPOND
\JMP\INI4\DID RESPOND - THAT'S OK
\IRS\0\STEP COUNTER
\JMP\INI3\TRY AGAIN
\HLT\\VDMC ISN'T RESPONDING
*
INI4\LDA*\INIT\GET RETURN ADDRESS
\SUB\=1\POINT AT 'JST INIT'
\STA\TMP1
ANOP\NOP\\A NOP INSTRUCTION
\LDA\ANOP
\STA*\TMP1\OVERWRITE 'JST INIT' WITH A 'NOP'
*
\LDA\MESS\PRINT START-UP MESSAGE
\JST\TYPE
\JST\PMSZ\CALCULATE MEMORY SIZE AND PRINT
*
\LDA\DINT\POINT AT INTERRUPT ROUTINE
\STA\STDI
\LDA\DMCM
\SMK\'20\SET STANDARD INTERRUPT MASK
\ENB\\ENABLE INTERRUPTS
\EJCT
*
*    INITIALIZE BUFFER STRUCTURES
*
\LDA\HIAD\GET HIGHEST ADDRESS
\SUB\PDBS\AVAILABLE FREE MEMORY
\STA\TMP1
\LDX\MBUF
IBS1\LDA\DBFE,1\GET NOMINAL BUFFER SIZE
\MPY\TMP1\SCALE BY AVAILABLE MEMORY SIZE
\SUB\BDSZ\SPACE FOR DESCRIPTOR
\STA\DBFE,1\SAVE MODIFIED SIZE
\IRS\0
\JMP\IBS1\LOOP OVER BUFFERS
*
\LDA\BDSZ\DESCRIPTOR SIZE
\MPY\NBUF\TIMES NUMBER OF BUFFERS
\IAB\\RECOVER LSBS
\ADD\PDBS\START OF BUFFER DESCRIPTORS
\STA\0\POINT X PAST LAST DESCRIPTOR
\LDA\MBUF\INITIALISE
\STA\TMP1\BUFFER COUNTER
\LDA\HIAD\HIGHEST ADDRESS
\AOA\\PLUS ONE
\STA\TMP2\I.E. POINT PAST LAST BUFFER
\LDA\PDBE
\STA\TMP3\POINT TO DMC BUFFER TABLE
\JMP\IBSX
\EJCT
************************************************************
*
*   START-UP MESSAGE
*
MESS\DAC\*+1
\DEC\-19\SET TO -VE (BCI-COUNT + 3)
\OCT\106612,106612
*\  \   11223344556677889900112233445566
\BCI\16,DMC TEST, 11 NOVEMBER 2017 REV A 
\OCT\106612
************************************************************
*   MEMORY SIZE MESSAGE
*
MSZM\DAC\*+1
\DEC\-9
\BCI\6,MEMORY SIZE 
MSZX\OCT\0  
\BCI\1,K 
\OCT\106612
\EJCT
************************************************************
*
*    DETERMINE HIGHEST ADDRESS IN MEMORY
*
HADR\DAC\**
\IFN\HSEA
\EXA\\EXTENDED ADDR. IF AVAILABLE
\ENDC
\LDA\PEND\POINT PAST END OF PROGRAM
\SKP\\ASSUMED MUCH LESS THAN '040000
HAD1\LDA\TMP1\GET ADDRESS
\ADD\='1000\STEP SECTOR
\ANA\='177000\START OF SECTOR
\STA\TMP1\SAVE ADDRESS
\STA*\TMP1\WRITE DATA INTO MEM THERE
\CRA\\CLEAR LOCATION ZERO
\STA\0\REACHED BY INDEXING OR INDIRECTION
\LDA*\TMP1\READ IT BACK
\ERA\TMP1\COMPARE
\SNZ\\HIGHEST ADDRESS REACHED?
\JMP\HAD1\IF NOT, DO ANOTHER TIME
\RMP\\RESET ANY ERRORS
\LDA\TMP1
\SUB\=1\HIGHEST ADDRESS
\STA\HIAD\SAVE
\JMP*\HADR
\EJCT
************************************************************
*
*    PRINT MEMORY SIZE
*
PMSZ\DAC\**
\CRA
\STA\MSZX
\JST\HADR
\AOA\\ONE BEYOND HIGHEST ADDRESS
\LGR\10\IN K WORDS
M10L\CAS\=10
\NOP\\>10
\SKP\\=10
\JMP\M01S\DONE WITH THE TENS
\SUB\=10
\IMA\MSZX
\ADD\='400\INCREMENT MSB
\IMA\MSZX
\JMP\M10L
*
M01S\ADD\MSZX
\CAS\='400
\NOP\\IF >= 1 IN MSB THEN ADD DIFFERENCE
\ADD\='10000\BETWEEN '0' AND ' '
\ADD\='120260\ADD ' ' TO MSB, '0' TO LSB
\STA\MSZX
\LDA\MSZM
\JST\TYPE
\JMP*\PMSZ
*
\EJCT
************************************************************
*    THE END...
\FIN
AEND\EQU\*
\END\STRT
$
************************************************************
* XEMACS LOCAL VARIABLES
*
* LOCAL VARIABLES:
* MODE: FUNDAMENTAL
* END:
